@@ -35,8 +35,6 @@
 #include <interfaces/iplugin.h>
 #include <interfaces/istatus.h>
 
-#include "gdbcontroller.h"
-
 class QLabel;
 class QMenu;
 class QDBusInterface;
@@ -58,11 +56,10 @@ class GDBBreakpointWidget;
 class FramestackWidget;
 class DisassembleWidget;
 class Breakpoint;
-class GDBController;
-class VariableWidget;
 class GDBOutputWidget;
 class ViewerWidget;
 class DebugSession;
+template<typename T> class DebuggerToolFactory;
 
 class CppDebuggerPlugin : public KDevelop::IPlugin, public KDevelop::IStatus
 {
@@ -75,6 +72,8 @@ public:
     CppDebuggerPlugin( QObject *parent, const QVariantList & = QVariantList() );
     ~CppDebuggerPlugin();
 
+    virtual void unload();
+    
     virtual KDevelop::ContextMenuExtension contextMenuExtension( KDevelop::Context* );
     
     DebugSession *createSession();
@@ -151,8 +150,10 @@ private:
     KConfigGroup m_config;
 
     DebugSession* m_session;
-
-    GDBController* m_controller;
+    DebuggerToolFactory< DisassembleWidget >* disassemblefactory;
+    DebuggerToolFactory< GDBOutputWidget >* gdbfactory;
+    DebuggerToolFactory< ViewerWidget >* viewerfactory;
+    
 };
 
 }
