@@ -25,6 +25,8 @@
 namespace KDevelop {
 class TestCore;
 }
+namespace GDBDebugger {
+
 class GdbTest : public QObject
 {
     Q_OBJECT
@@ -52,11 +54,22 @@ private Q_SLOTS:
     void testInsertBreakpointFunctionName();
     void testShowStepInSource();
     void testStack();
+    void testStackFetchMore();
+    void testStackDeactivateAndActive();
+    void testStackSwitchThread();
     void testAttach();
     void testCoreFile();
+    void testVariablesLocals();
+    void testVariablesLocalsStruct();
+    void testVariablesWatches();
+    void testVariablesWatchesTwoSessions();
+    void testVariablesStopDebugger();
+    void testVariablesStartSecondSession();
 private:
     KDevelop::TestCore* m_core;
-    void waitForState(const GDBDebugger::DebugSession& session, KDevelop::IDebugSession::DebuggerState state, const char *file, int line);
+    void waitForState(GDBDebugger::DebugSession *session, KDevelop::IDebugSession::DebuggerState state, const char *file, int line);
 };
 
+}
+
 #endif // GDBTEST_H
