@@ -32,7 +32,6 @@
 #include <language/duchain/declaration.h>
 #include <language/duchain/classfunctiondeclaration.h>
 #include <language/duchain/namespacealiasdeclaration.h>
-#include "../cppduchain/navigation/navigationwidget.h"
 #include <language/duchain/duchainutils.h>
 #include <language/duchain/classdeclaration.h>
 #include "../cppduchain/qtfunctiondeclaration.h"
@@ -200,8 +199,8 @@ KTextEditor::CodeCompletionModel::CompletionProperties NormalDeclarationCompleti
   return p;
 }
 
-bool declarationNeedsTemplateParameters(Declaration* decl) {
-  Cpp::TemplateDeclaration* asTemplate = dynamic_cast<Cpp::TemplateDeclaration*>(decl);
+bool declarationNeedsTemplateParameters(const Declaration* decl) {
+  const Cpp::TemplateDeclaration* asTemplate = dynamic_cast<const Cpp::TemplateDeclaration*>(decl);
   if(asTemplate) {
     DUContext* templateContext = asTemplate->templateContext(decl->topContext());
     if(templateContext) {
@@ -214,9 +213,15 @@ bool declarationNeedsTemplateParameters(Declaration* decl) {
   return false;
 }
 
+
+bool NormalDeclarationCompletionItem::completingTemplateParameters() const
+{
+  return m_isTemplateCompletion || declarationNeedsTemplateParameters(m_declaration.data());
+}
+
 QString NormalDeclarationCompletionItem::shortenedTypeString(KDevelop::DeclarationPointer decl, int desiredTypeLength) const
 {
-  if(m_cachedTypeStringDecl == decl && m_cachedTypeStringLength == desiredTypeLength)
+  if(m_cachedTypeStringDecl == decl && m_cachedTypeStringLength == static_cast<uint>(desiredTypeLength))
     return m_cachedTypeString;
   
   QString ret;
@@ -418,7 +423,7 @@ QVariant NormalDeclarationCompletionItem::data(const QModelIndex& index, int rol
         {
           QString ret;
           
-          if(m_isTemplateCompletion || declarationNeedsTemplateParameters(dec))
+          if(completingTemplateParameters())
             createTemplateArgumentList(*this, ret, 0);
           
           if (dec->type<FunctionType>()) {
@@ -521,59 +526,6 @@ KSharedPtr<CodeCompletionContext> NormalDeclarationCompletionItem::completionCon
   return KSharedPtr<CodeCompletionContext>::staticCast(m_completionContext);
 }
 
-
-int IncludeFileCompletionItem::inheritanceDepth() const
-{
-  return includeItem.pathNumber;
-}
-
-int IncludeFileCompletionItem::argumentHintDepth() const
-{
-  return 0;
-}
-
-QVariant IncludeFileCompletionItem::data(const QModelIndex& index, int role, const KDevelop::CodeCompletionModel* model) const
-{
-  DUChainReadLocker lock(DUChain::lock(), 500);
-  if(!lock.locked()) {
-    kDebug(9007) << "Failed to lock the du-chain in time";
-    return QVariant();
-  }
-
-  const Cpp::IncludeItem& item( includeItem );
-
-  switch (role) {
-    case CodeCompletionModel::IsExpandable:
-      return QVariant(true);
-    case CodeCompletionModel::ExpandingWidget: {
-      Cpp::NavigationWidget* nav = new Cpp::NavigationWidget(item, model->currentTopContext());
-      model->addNavigationWidget(this, nav);
-
-       QVariant v;
-       v.setValue<QWidget*>((QWidget*)nav);
-       return v;
-    }
-    case Qt::DisplayRole:
-      switch (index.column()) {
-        case CodeCompletionModel::Prefix:
-          if(item.isDirectory)
-            return QVariant("directory");
-          else
-            return QVariant("file");
-        case CodeCompletionModel::Name: {
-          return item.isDirectory ? item.name + '/' : item.name;
-        }
-      }
-      break;
-    case CodeCompletionModel::ItemSelected:
-    {
-      return QVariant( Cpp::NavigationWidget::shortDescription(item) );
-    }
-  }
-
-  return QVariant();
-}
-
 void IncludeFileCompletionItem::execute(KTextEditor::Document* document, const KTextEditor::Range& _word) {
 
   KTextEditor::Range word(_word);
@@ -588,7 +540,7 @@ void IncludeFileCompletionItem::execute(KTextEditor::Document* document, const K
       if(lineText.startsWith('"'))
         newText += '\"';
       else if(lineText.startsWith('<'))
-        newText += '>';
+       newText += '>';
     }
 
     word.end().setColumn( document->lineLength(word.end().line()) );
