@@ -201,10 +201,11 @@ class AdaptSignatureAction : public KDevelop::IAssistantAction {
         return;
       }
 
+      ///@todo Exactly watch what components are changed and how
       ///@todo Handle return-type
       ///@todo Keep default-parameters
       ///@todo Do matching of the arguments
-      ///@todo Eventually do real renaming
+      ///@todo Eventually do real refactoring-like renaming
       if(!functionContext || functionContext->type() != DUContext::Function) {
         kDebug() << "no correct function context";
         return;
@@ -215,10 +216,11 @@ class AdaptSignatureAction : public KDevelop::IAssistantAction {
       }
       
       DocumentChangeSet changes;
-      DocumentChangePointer change(new DocumentChange(functionContext->url(), functionContext->range(), QString(), makeSignatureString(m_newSignature, m_otherSideContext.data())));
-      change->m_ignoreOldText = true;
+      DocumentChange change(functionContext->url(), functionContext->range(), QString(), makeSignatureString(m_newSignature, m_otherSideContext.data()));
+      change.m_ignoreOldText = true;
       changes.addChange( change );
-      DocumentChangeSet::ChangeResult result = changes.applyAllChanges(DocumentChangeSet::WarnOnFailedChange);
+      changes.setReplacementPolicy(DocumentChangeSet::WarnOnFailedChange);
+      DocumentChangeSet::ChangeResult result = changes.applyAllChanges();
       if(!result) {
         KMessageBox::error(0, i18n("Failed to apply changes: %1", result.m_failureReason));
       }
