@@ -101,7 +101,14 @@ uint buildIdentifierForType(AbstractType::Ptr type, IndexedTypeIdentifier& id, u
       id.setIdentifier(idType->qualifiedIdentifier());
   }else{
     //Just create it as an expression
-    id.setIdentifier(QualifiedIdentifier(type->toString(), true));
+    AbstractType::Ptr useTypeText = type;
+    if(type->modifiers() & AbstractType::ConstModifier)
+    {
+      //Remove the 'const' modifier, as it will be added to the type-identifier below
+      useTypeText = type->indexed().abstractType();
+      useTypeText->setModifiers(useTypeText->modifiers() & (~AbstractType::ConstModifier));
+    }
+    id.setIdentifier(QualifiedIdentifier(useTypeText->toString(), true));
   }
   if(type->modifiers() & AbstractType::ConstModifier)
     id.setIsConstant(true);
@@ -706,6 +713,7 @@ IndexedTypeIdentifier shortenedTypeIdentifier(AbstractType::Ptr type, DUContext*
   if(type.cast<DelayedType>())
     identifier = type.cast<DelayedType>()->identifier();
   identifier = stripPrefixIdentifiers(identifier, stripPrefix);
+
   if(isReference) {
     identifier.setIsReference(true);
     if(isConstReference)
@@ -745,5 +753,34 @@ bool isFriend(KDevelop::Declaration* _class, KDevelop::Declaration* _friend) {
   return false;
 }
 
+DUContext* getTemplateContext(DUContext* internal, const TopDUContext* source) {
+  if(internal->type() == DUContext::Template)
+    return internal;
+  
+  if(!source)
+    source = internal->topContext();
+  
+  foreach( const DUContext::Import &ctx, internal->importedParentContexts() ) {
+    DUContext* c = ctx.context(source);
+    if( c ) {
+      if( c->type() == DUContext::Template )
+        return c;
+      c = getTemplateContext(c, source);
+      if(c)
+        return c;
+    }
+  }
+  return 0;
+}
+
+///Returns the context assigned to the given declaration that contains the template-parameters, if available. Else zero.
+DUContext* getTemplateContext(Declaration* decl, const TopDUContext* source) {
+  DUContext* internal = decl->internalContext();
+  if( !internal )
+    return 0;
+  
+  return getTemplateContext(internal, source);
+}
+
 }
 
