@@ -82,21 +82,6 @@ ClassDeclarationData::ClassType classTypeFromTokenKind(int kind)
   }
 }
 
-///Returns the context assigned to the given declaration that contains the template-parameters, if available. Else zero.
-DUContext* getTemplateContext(Declaration* decl) {
-  DUContext* internal = decl->internalContext();
-  if( !internal )
-    return 0;
-  if(internal->type() == DUContext::Template)
-    return internal;
-  foreach( const DUContext::Import &ctx, internal->importedParentContexts() ) {
-    if( ctx.context(decl->topContext()) )
-      if( ctx.context(decl->topContext())->type() == DUContext::Template )
-        return ctx.context(decl->topContext());
-  }
-  return 0;
-}
-
 bool DeclarationBuilder::changeWasSignificant() const
 {
   ///@todo Also set m_changeWasSignificant if publically visible declarations were removed(needs interaction with abstractcontextbuilder)
@@ -202,13 +187,17 @@ void DeclarationBuilder::visitFunctionDeclaration(FunctionDefinitionAST* node)
   parseFunctionSpecifiers(node->function_specifiers);
   
   //Used to map to the top level function node once the Declaration is built
-  m_declarationNode = node;
-
+  if(m_mapAstDuChain)
+    m_mappedNodes.push(node);
+  
   m_functionDefinedStack.push(node->start_token);
 
   DeclarationBuilderBase::visitFunctionDeclaration(node);
 
   m_functionDefinedStack.pop();
+  
+  if(m_mapAstDuChain)
+    m_mappedNodes.pop();
 
   popSpecifiers();
 }
@@ -255,13 +244,17 @@ void DeclarationBuilder::visitSimpleDeclaration(SimpleDeclarationAST* node)
   parseStorageSpecifiers(node->storage_specifiers);
   parseFunctionSpecifiers(node->function_specifiers);
 
-  m_declarationNode = node;
+  if(m_mapAstDuChain)
+    m_mappedNodes.push(node);
   
   m_functionDefinedStack.push(0);
 
   DeclarationBuilderBase::visitSimpleDeclaration(node);
 
   m_functionDefinedStack.pop();
+  
+  if(m_mapAstDuChain)
+    m_mappedNodes.pop();
 
   popSpecifiers();
 }
@@ -286,11 +279,8 @@ void DeclarationBuilder::visitDeclarator (DeclaratorAST* node)
     Declaration* decl = openFunctionDeclaration(node->id, node);
     
     ///Create mappings iff the AST feature is specified
-    if(m_mapAstDuChain && m_declarationNode)
-    {
-      editor()->parseSession()->mapAstDuChain(m_declarationNode, KDevelop::DeclarationPointer(decl));
-      m_declarationNode = 0;
-    }
+    if(m_mapAstDuChain && !m_mappedNodes.empty())
+      editor()->parseSession()->mapAstDuChain(m_mappedNodes.top(), KDevelop::DeclarationPointer(decl));
 
     if( !m_functionDefinedStack.isEmpty() ) {
         DUChainWriteLocker lock(DUChain::lock());
@@ -645,11 +635,8 @@ Declaration* DeclarationBuilder::openDefinition(NameAST* name, AST* rangeNode, b
   Declaration* ret = openNormalDeclaration(name, rangeNode, KDevelop::Identifier(), collapseRange);
   
   ///Create mappings iff the AST feature is specified
-  if(m_mapAstDuChain && m_declarationNode)
-  {
-    editor()->parseSession()->mapAstDuChain(m_declarationNode, KDevelop::DeclarationPointer(ret));
-    m_declarationNode = 0;
-  }
+  if(m_mapAstDuChain && !m_mappedNodes.empty())
+    editor()->parseSession()->mapAstDuChain(m_mappedNodes.top(), KDevelop::DeclarationPointer(ret));
 
   DUChainWriteLocker lock(DUChain::lock());
   ret->setDeclarationIsDefinition(true);
@@ -742,7 +729,7 @@ void DeclarationBuilder::closeDeclaration(bool forceInstance)
       
     if (lastType()) {
 
-      AbstractType::Ptr type = lastType();
+      AbstractType::Ptr type = typeForCurrentDeclaration();
       IdentifiedType* idType = dynamic_cast<IdentifiedType*>(type.unsafeData());
       DelayedType::Ptr delayed = type.cast<DelayedType>();
 
@@ -761,7 +748,7 @@ void DeclarationBuilder::closeDeclaration(bool forceInstance)
           currentDeclaration()->setKind(Declaration::Type);
       }
 
-      currentDeclaration()->setType(lastType());
+      currentDeclaration()->setType(type);
     }else{
       currentDeclaration()->setAbstractType(AbstractType::Ptr());
     }
@@ -1440,7 +1427,11 @@ void DeclarationBuilder::parseFunctionSpecifiers(const ListNode<std::size_t>* fu
   m_functionSpecifiers.push(specs);
 }
 
-void DeclarationBuilder::visitParameterDeclaration(ParameterDeclarationAST* node) {
+void DeclarationBuilder::visitParameterDeclaration(ParameterDeclarationAST* node)
+{
+  if(m_mapAstDuChain)
+    m_mappedNodes.push(node);
+  
   DeclarationBuilderBase::visitParameterDeclaration(node);
   
   AbstractFunctionDeclaration* function = currentDeclaration<AbstractFunctionDeclaration>();
@@ -1460,6 +1451,9 @@ void DeclarationBuilder::visitParameterDeclaration(ParameterDeclarationAST* node
       closeDeclaration();
     }
   }
+  
+  if(m_mapAstDuChain)
+    m_mappedNodes.pop();
 }
 
 
