@@ -41,12 +41,13 @@
 #include "qtfunctiondeclaration.h"
 #include "missingdeclarationtype.h"
 #include "missingdeclarationproblem.h"
+#include "dumpchain.h"
 
 //If this is enabled and a type is not found, it is searched again with verbose debug output.
 //#define DEBUG_RESOLUTION_PROBLEMS
 
 //If this is enabled, all encounterd problems will be dumped to kDebug
-//#define DUMP_PROBLEMS
+// #define DUMP_PROBLEMS
 
 //If this is enabled, problems will be created when no overloaded function was found for a function-call. This is expensive,
 //because the problem report contains a lot of information, and the problem currently appears very often.
@@ -252,7 +253,8 @@ void ExpressionVisitor::problem( AST* node, const QString& str ) {
   kDebug(9007) << "Cpp::ExpressionVisitor problem:" << str;
 
   kDebug(9007) << "Cpp::ExpressionVisitor dumping the node that created the problem";
-  DumpChain d;
+  Cpp::DumpChain d;
+  
   d.dump(node, m_session);
 #endif
 }
@@ -482,7 +484,7 @@ void ExpressionVisitor::findMember( AST* node, AbstractType::Ptr base, const Ide
 
     clearLast();
 
-    NameASTVisitor nameV( m_session, this, searchInContext, topContext(), position.isValid() ? position : searchInContext->range().end, m_memberAccess ? DUContext::DontSearchInParent : DUContext::NoSearchFlags );
+    NameASTVisitor nameV( m_session, this, searchInContext, topContext(), m_currentContext, position.isValid() ? position : searchInContext->range().end, m_memberAccess ? DUContext::DontSearchInParent : DUContext::NoSearchFlags );
     nameV.run(node, m_skipLastNamePart);
 
     if( nameV.identifier().isEmpty() ) {
@@ -509,6 +511,15 @@ void ExpressionVisitor::findMember( AST* node, AbstractType::Ptr base, const Ide
       m_lastDeclarations = nameV.declarations();
 
       if( m_lastDeclarations.isEmpty() || !m_lastDeclarations.first().data() ) {
+        
+        if(Cpp::isTemplateDependent(m_currentContext) ) {
+          if(m_memberAccess || (node->qualified_names && nameV.foundSomething() && Cpp::isTemplateDependent(nameV.foundSomething().data()))) {
+          //Do nothing. Within a not instantiated template, we cannot be that sure
+          m_lastType.clear();
+          return;
+          }
+        }
+        
         MissingDeclarationType::Ptr missing(new MissingDeclarationType);
         
         missing->setIdentifier(IndexedTypeIdentifier(nameV.identifier()));
@@ -567,7 +578,7 @@ void ExpressionVisitor::findMember( AST* node, AbstractType::Ptr base, const Ide
       return; //We had a string-literal
     }
 
-    if( !node->literal && !node->sub_expression && !node->expression_statement && !node->name )
+    if( !node->sub_expression && !node->expression_statement && !node->name )
     {
       IndexedString startNumber = IndexedString::fromIndex(m_session->contents()[tokenFromIndex(node->start_token).position]); //Extracts the first digit
 
@@ -1040,7 +1051,7 @@ void ExpressionVisitor::createDelayedType( AST* node , bool expression ) {
 
     clearLast();
 
-    TypeASTVisitor comp(m_session, this, m_currentContext, topContext());
+    TypeASTVisitor comp(m_session, this, m_currentContext, topContext(), m_currentContext);
     comp.run(ast);
 
     LOCKDUCHAIN;
@@ -1579,25 +1590,45 @@ void ExpressionVisitor::createDelayedType( AST* node , bool expression ) {
 
     /**
      * If a class name was found, get its constructors.
-     *
-     * @todo Think uses should be built for typedef class constructors.
-     *       Normally, it should be a use of the typedef class, so refactoring can work
-     *       correctly. However we also want to know which constructor was called in other places.
      * */
     CppClassType::Ptr constructedType;
 
     AbstractType::Ptr oldLastType = m_lastType;
     
-    {
+    if(!m_lastInstance) {
       LOCKDUCHAIN;
+      if(m_lastDeclarations.isEmpty() && m_lastType && !m_lastInstance) {
+        IdentifiedType* idType = dynamic_cast<IdentifiedType*>(m_lastType.unsafeData());
+        if(idType) {
+          Declaration* decl = idType->declaration(m_source);
+          if(decl)
+            m_lastDeclarations << DeclarationPointer(decl);
+        }
+      }
+      
       if( !m_lastDeclarations.isEmpty() && m_lastDeclarations.first().data() && m_lastDeclarations.first()->kind() == Declaration::Type && (constructedType = unAliasedType(m_lastDeclarations.first()->logicalDeclaration(topContext())->abstractType()).cast<CppClassType>()) ) {
 
         if( constructedType && constructedType->declaration(topContext()) && constructedType->declaration(topContext())->internalContext() )
         {
           Declaration* constructedDecl = constructedType->declaration(topContext());
+          
+          //Replace a type with its constructros if there is constructors available, so overload-resolution can happen
           m_lastDeclarations = convert(constructedDecl->internalContext()->findLocalDeclarations( constructedDecl->identifier(), constructedDecl->internalContext()->range().end, topContext(), AbstractType::Ptr(), DUContext::OnlyFunctions ));
         }
       }
+    }else{
+      if(m_lastDeclarations.isEmpty() && m_lastType) {
+        LOCKDUCHAIN;
+        //The function-call operator may also be applied on instances that don't have an explicit declaration, like return-values
+        AbstractType::Ptr type = realType(m_lastType);
+        IdentifiedType* identified = dynamic_cast<IdentifiedType*>(type.unsafeData());
+        if(identified) {
+          DeclarationPointer decl(identified->declaration(m_source));
+          if(decl) {
+            m_lastDeclarations << decl;
+          }
+        }
+      }
     }
 
     /**
@@ -1605,12 +1636,12 @@ void ExpressionVisitor::createDelayedType( AST* node , bool expression ) {
      * node->arguments contains them, using recursive binary expressions
      * */
     QList<DeclarationPointer> declarations = m_lastDeclarations;
+    Instance oldInstance = m_lastInstance;
 
     QList<OverloadResolver::Parameter> oldParams = m_parameters;
     KDevVarLengthArray<AST*> oldParameterNodes = m_parameterNodes;
     m_parameters.clear();
     m_parameterNodes.clear();
-    //kDebug(9007) << "clearing parameters";
 
     //Backup the current use and invalidate it, we will update and create it after overload-resolution
     CurrentUse oldCurrentUse = m_currentUse;
@@ -1623,12 +1654,10 @@ void ExpressionVisitor::createDelayedType( AST* node , bool expression ) {
     if( m_lastType ) {
       m_parameters << OverloadResolver::Parameter( m_lastType, isLValue( m_lastType, m_lastInstance ) );
       m_parameterNodes.append(node->arguments);
-      //LOCKDUCHAIN;
-      //kDebug(9007) << "adding last parameter: " << (m_lastType.data() ? m_lastType->toString() : QString("<notype>"));
     }
 
     if( declarations.isEmpty() && !constructedType ) {
-      
+
       if(MissingDeclarationType::Ptr missing = oldLastType.cast<Cpp::MissingDeclarationType>()) {
         missing->arguments = m_parameters;
         missing->isFunction = true;
@@ -1655,20 +1684,23 @@ void ExpressionVisitor::createDelayedType( AST* node , bool expression ) {
       m_lastType = AbstractType::Ptr(constructedType.unsafeData());
       DeclarationPointer decl(constructedType->declaration(topContext()));
       m_lastInstance = Instance(decl.data());
+      m_lastDeclarations.clear();
+//       m_lastDeclarations << decl;
       lock.unlock();
-      if(oldCurrentUse.isValid)
+      if(oldCurrentUse.isValid) {
         newUse( oldCurrentUse.node, oldCurrentUse.start_token, oldCurrentUse.end_token, decl );
+      }
+      flushUse();
       return;
     }
 
     //Resolve functions
     DeclarationPointer chosenFunction;
     KDevelop::DUContextPointer ptr(m_currentContext);
-    OverloadResolver resolver( ptr, KDevelop::TopDUContextPointer(topContext()) );
+    OverloadResolver resolver( ptr, KDevelop::TopDUContextPointer(topContext()), oldInstance );
 
-    if( !fail ) {
+    if( !fail )
       chosenFunction = resolver.resolveList(m_parameters, convert(declarations));
-    }
 
     if( !chosenFunction && !m_strict ) {
       //Because we do not want to rely too much on our understanding of the code, we take the first function instead of totally failing.
@@ -1698,8 +1730,6 @@ void ExpressionVisitor::createDelayedType( AST* node , bool expression ) {
       chosenFunction = declarations.front();
     }
 
-    //kDebug(9007) << "Resetting old parameters of size " << oldParams.size();
-
     clearLast();
 
     if( constructedType ) {
@@ -1719,7 +1749,6 @@ void ExpressionVisitor::createDelayedType( AST* node , bool expression ) {
     static IndexedString functionCallOperatorIdentifier("operator()");
     
     bool createUseOnParen = (bool)chosenFunction && (constructedType || chosenFunction->identifier().identifier() == functionCallOperatorIdentifier);
-
     //Re-create the use we have discarded earlier, this time with the correct overloaded function chosen.
     lock.unlock();
     
@@ -1735,6 +1764,8 @@ void ExpressionVisitor::createDelayedType( AST* node , bool expression ) {
     m_parameterNodes = oldParameterNodes;
     m_parameters = oldParams;
     
+    flushUse();
+    
     if( m_lastType )
       expressionType( node, m_lastType, m_lastInstance );
   }
@@ -1776,7 +1807,7 @@ void ExpressionVisitor::createDelayedType( AST* node , bool expression ) {
       SimpleCursor position = container->range().end;
       lock.unlock();
       //This builds the uses
-      NameASTVisitor nameV( m_session, this, container, topContext(), position );
+      NameASTVisitor nameV( m_session, this, container, topContext(), m_currentContext, position );
       nameV.run(node->name, true);
       lock.lock();
     }
@@ -2011,6 +2042,28 @@ void ExpressionVisitor::createDelayedType( AST* node , bool expression ) {
     ///@todo respect const etc.
   }
 
+  void ExpressionVisitor::visitMemInitializer(MemInitializerAST* node)  {
+    PushPositiveContext pushContext( m_currentContext, node->ducontext );
+    {
+      LOCKDUCHAIN;
+      Declaration* klass = localClassFromCodeContext(m_currentContext);
+      if(klass)
+        m_lastType = klass->abstractType();
+    }
+    m_memberAccess = true;
+    visit(node->initializer_id);
+    m_memberAccess = false;
+    
+    AbstractType::Ptr itemType = m_lastType;
+    visit(node->expression);
+    TypePtr< MissingDeclarationType > missingDeclType = itemType.cast<MissingDeclarationType>();
+    if(m_lastType && missingDeclType) {
+        Cpp::ExpressionEvaluationResult res;
+        res.type = m_lastType->indexed();
+        res.isInstance = m_lastInstance;
+        missingDeclType->assigned = res;
+    }
+  }
 
   ///Nodes that are invalid inside an expression:
   void ExpressionVisitor::visitPtrToMember(PtrToMemberAST* node)  { problem(node, "node-type cannot be parsed"); }
@@ -2038,7 +2091,6 @@ void ExpressionVisitor::createDelayedType( AST* node , bool expression ) {
   void ExpressionVisitor::visitLabeledStatement(LabeledStatementAST* node)  { problem(node, "node-type cannot be parsed"); }
   void ExpressionVisitor::visitLinkageBody(LinkageBodyAST* node)  { problem(node, "node-type cannot be parsed"); }
   void ExpressionVisitor::visitLinkageSpecification(LinkageSpecificationAST* node)  { problem(node, "node-type cannot be parsed"); }
-  void ExpressionVisitor::visitMemInitializer(MemInitializerAST* node)  { problem(node, "node-type cannot be parsed"); }
   void ExpressionVisitor::visitNamespace(NamespaceAST* node)  { problem(node, "node-type cannot be parsed"); }
   void ExpressionVisitor::visitNamespaceAliasDefinition(NamespaceAliasDefinitionAST* node)  { problem(node, "node-type cannot be parsed"); }
   void ExpressionVisitor::visitParameterDeclaration(ParameterDeclarationAST* node)  { problem(node, "node-type cannot be parsed"); }
