@@ -68,6 +68,7 @@
 #include <language/duchain/topducontext.h>
 #include <language/duchain/smartconverter.h>
 #include <language/duchain/functiondefinition.h>
+#include <language/codegen/coderepresentation.h>
 #include <interfaces/contextmenuextension.h>
 
 #include "preprocessjob.h"
@@ -85,7 +86,9 @@
 #include "veritas/testswitch.h"
 #include "veritas/stubcontextaction.h"
 #include "veritas/uutcontextaction.h"
+#include "codegen/codeassistant.h"
 #include "codegen/cppnewclass.h"
+#include "codegen/makeimplementationprivate.h"
 
 #include "includepathresolver.h"
 #include "setuphelpers.h"
@@ -106,17 +109,7 @@
 #define DEBUG_UI_LOCKUP
 #define LOCKUP_INTERVAL 5
 #endif
-
-//List of possible headers used for definition/declaration fallback switching
-QStringList headerExtensions(QString("h,H,hh,hxx,hpp,tlh,h++").split(','));
-QStringList sourceExtensions(QString("c,cc,cpp,c++,cxx,C,m,mm,M,inl,_impl.h").split(','));
-
-QString addDot(QString ext) {
-  if(ext.contains('.')) //We need this check because of the _impl.h thing
-    return ext;
-  else
-    return "." + ext;
-}
+#include "cpputils.h"
 
 KTextEditor::Cursor normalizeCursor(KTextEditor::Cursor c) {
   c.setColumn(0);
@@ -148,18 +141,23 @@ Declaration* definitionForCursorDeclaration(const KDevelop::SimpleCursor& cursor
   return 0;
 }
 
-QStringList CppLanguageSupport::standardIncludePaths() const {
-  return *m_standardIncludePaths;
-}
-
+// For unit-tests that compile cpplanguagesupport.cpp into their executable
+// don't create the factories as that means 2 instances of the factory
+#ifndef BUILD_TESTS
 K_PLUGIN_FACTORY(KDevCppSupportFactory, registerPlugin<CppLanguageSupport>(); )
 K_EXPORT_PLUGIN(KDevCppSupportFactory(KAboutData("kdevcppsupport","kdevcpp", ki18n("C++ Support"), "0.1", ki18n("Support for C++ Language"), KAboutData::License_GPL)))
+#else
+class KDevCppSupportFactory : public KPluginFactory
+{
+public:
+    static KComponentData componentData() { return KComponentData(); };
+};
+#endif
 
 CppLanguageSupport::CppLanguageSupport( QObject* parent, const QVariantList& /*args*/ )
     : KDevelop::IPlugin( KDevCppSupportFactory::componentData(), parent ),
       KDevelop::ILanguageSupport(),
-      m_standardMacros(0),
-      m_standardEnvironment(0)
+      m_standardMacros(0)
 {
     m_self = this;
 
@@ -169,8 +167,6 @@ CppLanguageSupport::CppLanguageSupport( QObject* parent, const QVariantList& /*a
     m_highlights = new CppHighlighting( this );
     m_cc = new KDevelop::CodeCompletion( this, new Cpp::CodeCompletionModel(0), name() );
     m_missingIncludeCompletion = new KDevelop::CodeCompletion( this, new Cpp::MissingIncludeCompletionModel(0), name() );
-    m_standardMacros = new Cpp::ReferenceCountedMacroSet;
-    m_standardIncludePaths = new QStringList;
     
     Cpp::EnvironmentManager::setSimplifiedMatching(true);
     
@@ -179,19 +175,6 @@ CppLanguageSupport::CppLanguageSupport( QObject* parent, const QVariantList& /*a
 //     Cpp::EnvironmentManager::setMatchingLevel(Cpp::EnvironmentManager::Full);
 
     m_includeResolver = new CppTools::IncludePathResolver;
-    // Retrieve the standard include paths & macro definitions for this machine.
-    // Uses gcc commands to retrieve the information.
-    CppTools::setupStandardIncludePaths(*m_standardIncludePaths);
-    CppTools::setupStandardMacros(*m_standardMacros);
-    
-    m_standardEnvironment = PreprocessJob::createStandardEnvironment();
-
-    connect( core()->projectController(),
-             SIGNAL( projectOpened( KDevelop::IProject* ) ),
-             this, SLOT( projectOpened( KDevelop::IProject* ) ) );
-    connect( core()->projectController(),
-             SIGNAL( projectClosing( KDevelop::IProject* ) ),
-             this, SLOT( projectClosing( KDevelop::IProject* ) ) );
 
     m_quickOpenDataProvider = new IncludeFileDataProvider();
 
@@ -212,6 +195,11 @@ CppLanguageSupport::CppLanguageSupport( QObject* parent, const QVariantList& /*a
     KAction* newClassAction = actions->addAction("code_new_class");
     newClassAction->setText( i18n("Create &New Class") );
     connect(newClassAction, SIGNAL(triggered(bool)), this, SLOT(newClassWizard()));
+    
+//    KAction* pimplAction = actions->addAction("code_private_implementation");
+//    pimplAction->setText( i18n("Make Class Implementation Private") );
+//    pimplAction->setShortcut(Qt::ALT | Qt::META | Qt::Key_P);
+//    connect(pimplAction, SIGNAL(triggered(bool)), &SimpleRefactoring::self(), SLOT(executePrivateImplementationAction()));
 
     KAction* renameDeclarationAction = actions->addAction("code_rename_declaration");
     renameDeclarationAction->setText( i18n("Rename Declaration") );
@@ -225,7 +213,7 @@ CppLanguageSupport::CppLanguageSupport( QObject* parent, const QVariantList& /*a
     connect(moveIntoSourceAction, SIGNAL(triggered(bool)), &SimpleRefactoring::self(), SLOT(executeMoveIntoSourceAction()));
     
     Veritas::TestSwitch* ts = new Veritas::TestSwitch(this);
-    ts->setStandardMacros(m_standardMacros);
+    ts->setStandardMacros(const_cast<Cpp::ReferenceCountedMacroSet*>(&CppUtils::standardMacros()));
     ts->connectAction(actionCollection());
 
     m_stubAction = new Veritas::StubContextAction(this);
@@ -236,112 +224,8 @@ CppLanguageSupport::CppLanguageSupport( QObject* parent, const QVariantList& /*a
 #ifdef DEBUG_UI_LOCKUP
     m_blockTester = new UIBlockTester(LOCKUP_INTERVAL);
 #endif
-}
 
-bool CppLanguageSupport::isHeader(const KUrl &url) const {
-  QFileInfo fi( url.toLocalFile() );
-  QString path = fi.filePath();
-  // extract the exension
-  QString ext = fi.suffix();
-  if ( ext.isEmpty() )
-    return true;
-  
-  return headerExtensions.contains(ext);
-}
-
-KUrl CppLanguageSupport::sourceOrHeaderCandidate( const KUrl &url, bool fast ) const
-{
-  QString urlPath = url.toLocalFile(); ///@todo Make this work with real urls
-
-// get the path of the currently active document
-  QFileInfo fi( urlPath );
-  QString path = fi.filePath();
-  // extract the exension
-  QString ext = fi.suffix();
-  if ( ext.isEmpty() )
-    return KUrl();
-  // extract the base path (full path without '.' and extension)
-  QString base = path.left( path.length() - ext.length() - 1 );
-  //kDebug( 9007 ) << "base: " << base << ", ext: " << ext << endl;
-  // just the filename without the extension
-  QString fileNameWoExt = fi.fileName();
-  if ( !ext.isEmpty() )
-    fileNameWoExt.replace( "." + ext, "" );
-  QStringList possibleExts;
-  // depending on the current extension assemble a list of
-  // candidate files to look for
-  QStringList candidates;
-  // special case for template classes created by the new class dialog
-  if ( path.endsWith( "_impl.h" ) )
-  {
-    QString headerpath = path;
-    headerpath.replace( "_impl.h", ".h" );
-    candidates << headerpath;
-    fileNameWoExt.replace( "_impl", "" );
-    possibleExts << "h";
-  }
-  // if file is a header file search for implementation file
-  else if ( headerExtensions.contains( ext ) )
-  {
-    foreach(const QString& ext, sourceExtensions)
-      candidates << ( base + addDot(ext) );
-
-    possibleExts = sourceExtensions;
-  }
-  // if file is an implementation file, search for header file
-  else if ( sourceExtensions.contains( ext ) )
-  {
-    foreach(const QString& ext, headerExtensions)
-      candidates << ( base + addDot(ext) );
-
-    possibleExts = headerExtensions;
-  }
-  // search for files from the assembled candidate lists, return the first
-  // candidate file that actually exists or QString::null if nothing is found.
-  QStringList::ConstIterator it;
-  for ( it = candidates.constBegin(); it != candidates.constEnd(); ++it )
-  {
-//     kDebug( 9007 ) << "Trying " << ( *it ) << endl;
-    if ( QFileInfo( *it ).exists() )
-    {
-//       kDebug( 9007 ) << "using: " << *it << endl;
-      return * it;
-    }
-  }
-
-  if(fast)
-    return KUrl();
-
-  //kDebug( 9007 ) << "Now searching in project files." << endl;
-  // Our last resort: search the project file list for matching files
-  KUrl::List projectFileList;
-
-  foreach (KDevelop::IProject *project, core()->projectController()->projects()) {
-      if (project->inProject(url)) {
-        QList<ProjectFileItem*> files = project->files();
-        foreach(ProjectFileItem* file, files)
-          projectFileList << file->url();
-      }
-  }
-  QFileInfo candidateFileWoExt;
-  QString candidateFileWoExtString;
-  foreach ( const KUrl& url, projectFileList )
-  {
-    candidateFileWoExt.setFile(url.toLocalFile());
-    //kDebug( 9007 ) << "candidate file: " << url << endl;
-    if( !candidateFileWoExt.suffix().isEmpty() )
-      candidateFileWoExtString = candidateFileWoExt.fileName().replace( "." + candidateFileWoExt.suffix(), "" );
-
-    if ( candidateFileWoExtString == fileNameWoExt )
-    {
-      if ( possibleExts.contains( candidateFileWoExt.suffix() ) || candidateFileWoExt.suffix().isEmpty() )
-      {
-        //kDebug( 9007 ) << "checking if " << url << " exists" << endl;
-        return url;
-      }
-    }
-  }
-  return KUrl();
+    m_assistant = new Cpp::StaticCodeAssistant;
 }
 
 ///@todo Make this work again with non-class declarations/definitions
@@ -447,7 +331,7 @@ void CppLanguageSupport::switchDefinitionDeclaration()
 
   lock.unlock();
   ///- If no definition/declaration could be found to switch to, just switch the document using normal header/source heuristic by file-extension
-  KUrl url = sourceOrHeaderCandidate(doc->textDocument()->url());
+  KUrl url = CppUtils::sourceOrHeaderCandidate(doc->textDocument()->url());
 
   if(url.isValid()) {
     core()->documentController()->openDocument(url);
@@ -471,20 +355,11 @@ CppLanguageSupport::~CppLanguageSupport()
     core()->languageController()->backgroundParser()->clear(this);
 
 
-    delete m_standardMacros;
-    delete m_standardIncludePaths;
     delete m_includeResolver;
 #ifdef DEBUG_UI_LOCKUP
     delete m_blockTester;
 #endif
-}
-
-const Cpp::ReferenceCountedMacroSet& CppLanguageSupport::standardMacros() const {
-    return *m_standardMacros;
-}
-
-const ParsingEnvironment* CppLanguageSupport::standardEnvironment() const {
-  return m_standardEnvironment;
+    delete m_assistant;
 }
 
 CppLanguageSupport* CppLanguageSupport::self() {
@@ -501,44 +376,6 @@ const KDevelop::ICodeHighlighting *CppLanguageSupport::codeHighlighting() const
     return m_highlights;
 }
 
-void CppLanguageSupport::projectOpened(KDevelop::IProject *project)
-{
-    Q_UNUSED(project)
-    // FIXME Add signals slots from the filemanager for:
-    //       1. filesAddedToProject
-    //       2. filesRemovedFromProject
-    //       3. filesChangedInProject
-
-    ///@todo Be more clever about getting include-paths for header-files: They are not directly part of the project, but usually they have an assigned source-file, so use the include-path from that file. Then we won't need to do the ordering.
-
-    //Since there may be additional information like include-paths available now, reparse all open documents
-    QList<IDocument*> headers;
-    foreach(IDocument* doc, core()->documentController()->openDocuments()) {
-        if(project->inProject(doc->url())) {
-          bool isSource = false;
-          QString path = doc->url().toLocalFile();
-
-          foreach(const QString& str, sourceExtensions)
-            if(path.endsWith(str))
-              isSource = true;
-
-          if(isSource) //Add source-files first, because their include-paths may be important for headers
-            core()->languageController()->backgroundParser()->addDocument(doc->url());
-          else
-            headers << doc;
-        }
-    }
-
-    foreach(IDocument* doc, headers)
-      core()->languageController()->backgroundParser()->addDocument(doc->url());
-}
-
-void CppLanguageSupport::projectClosing(KDevelop::IProject *project)
-{
-    Q_UNUSED(project)
-    //TODO: Anything to do here?!?!
-}
-
 void CppLanguageSupport::findIncludePathsForJob(CPPParseJob* job)
 {
   IncludePathComputer* comp = new IncludePathComputer(KUrl(job->document().str()), job->preprocessorProblemsPointer());
@@ -546,134 +383,6 @@ void CppLanguageSupport::findIncludePathsForJob(CPPParseJob* job)
   job->gotIncludePaths(comp);
 }
 
-KUrl::List CppLanguageSupport::findIncludePaths(const KUrl& source, QList<KDevelop::ProblemPointer>* problems) const {
-  IncludePathComputer comp(source, problems);
-  comp.computeForeground();
-  comp.computeBackground();
-  return comp.result();
-}
-
-QList<Cpp::IncludeItem> CppLanguageSupport::allFilesInIncludePath(const KUrl& source, bool local, const QString& addPath, KUrl::List addIncludePaths, bool onlyAddedIncludePaths, bool prependAddedPathToName, bool allowSourceFiles) const {
-
-    QMap<KUrl, bool> hadPaths; //Only process each path once
-    QList<Cpp::IncludeItem> ret;
-
-    KUrl::List paths = addIncludePaths;
-    if(!onlyAddedIncludePaths) {
-      paths += findIncludePaths(source, 0);
-
-      if(local) {
-          KUrl localPath = source;
-          localPath.setFileName(QString());
-          paths.push_front(localPath);
-      }
-    }
-
-    int pathNumber = 0;
-
-    foreach(const KUrl& path, paths)
-    {
-        if(!hadPaths.contains(path)) {
-            hadPaths[path] = true;
-        }else{
-            continue;
-        }
-        if(!path.isLocalFile()) {
-            kDebug(9007) << "include-path " << path << " is not local";
-            continue;
-        }
-        KUrl searchPathUrl = path;
-        QString absoluteBase = searchPathUrl.toLocalFile();
-        searchPathUrl.addPath(addPath);
-        QString searchPath = searchPathUrl.toLocalFile();
-
-        QDirIterator dirContent(searchPath);
-
-        while(dirContent.hasNext()) {
-             dirContent.next();
-            Cpp::IncludeItem item;
-            item.name = dirContent.fileName();
-
-            if(item.name.startsWith('.') || item.name.endsWith("~")) //This filters out ".", "..", and hidden files, and backups
-              continue;
-            QString suffix = dirContent.fileInfo().suffix();
-            if(!dirContent.fileInfo().suffix().isEmpty() && !headerExtensions.contains(suffix) && (!allowSourceFiles || !sourceExtensions.contains(suffix)))
-              continue;
-            
-            if(prependAddedPathToName)
-              item.name = addPath + item.name;
-            
-            item.isDirectory = dirContent.fileInfo().isDir();
-            item.basePath = absoluteBase;
-            item.pathNumber = pathNumber;
-
-            ret << item;
-        }
-        ++pathNumber;
-    }
-
-    return ret;
-}
-
-QPair<KUrl, KUrl> CppLanguageSupport::findInclude(const KUrl::List& includePaths, const KUrl& localPath, const QString& includeName, int includeType, const KUrl& skipPath, bool quiet) const {
-    QPair<KUrl, KUrl> ret;
-#ifdef DEBUG
-    kDebug(9007) << "searching for include-file" << includeName;
-    if( !skipPath.isEmpty() )
-        kDebug(9007) << "skipping path" << skipPath;
-#endif
-
-    if (includeType == rpp::Preprocessor::IncludeLocal && localPath != skipPath) {
-      QString check = localPath.toLocalFile(KUrl::AddTrailingSlash) + includeName;
-        QFileInfo info(check);
-        if (info.exists() && info.isReadable() && info.isFile()) {
-            //kDebug(9007) << "found include file:" << info.absoluteFilePath();
-            ret.first = KUrl(info.absoluteFilePath());
-            ret.first.cleanPath();
-            ret.second = localPath;
-            return ret;
-        }
-    }
-
-    //When a path is skipped, we will start searching exactly after that path
-    bool needSkip = !skipPath.isEmpty();
-
-restart:
-    foreach( const KUrl& path, includePaths ) {
-        if( needSkip ) {
-            if( path == skipPath ) {
-                needSkip = false;
-                continue;
-            }
-        }
-
-        QString check = path.toLocalFile(KUrl::AddTrailingSlash) + includeName;
-        QFileInfo info(check);
-
-        if (info.exists() && info.isReadable() && info.isFile()) {
-            //kDebug(9007) << "found include file:" << info.absoluteFilePath();
-            ret.first = KUrl(info.absoluteFilePath());
-            ret.first.cleanPath();
-            ret.second = path.toLocalFile();
-            return ret;
-        }
-    }
-
-    if( needSkip ) {
-        //The path to be skipped was not found, so simply start from the begin, considering any path.
-        needSkip = false;
-        goto restart;
-    }
-
-    if( ret.first.isEmpty() && !quiet ) {
-        kDebug(9007) << "FAILED to find include-file" << includeName << "in paths:";
-        foreach( const KUrl& path, includePaths )
-            kDebug(9007) << path;
-    }
-    
-    return ret;
-}
-
 QString CppLanguageSupport::name() const
 {
     return "C++";
@@ -687,7 +396,7 @@ KDevelop::ILanguage *CppLanguageSupport::language()
 TopDUContext* CppLanguageSupport::standardContext(const KUrl& url, bool allowProxyContext)
 {
   DUChainReadLocker lock(DUChain::lock());
-  const ParsingEnvironment* env = standardEnvironment();
+  const ParsingEnvironment* env = PreprocessJob::standardEnvironment();
   KDevelop::TopDUContext* top;
   top = KDevelop::DUChain::self()->chainForDocument(url, env, Cpp::EnvironmentManager::isSimplifiedMatching());
 
@@ -709,7 +418,9 @@ TopDUContext* CppLanguageSupport::standardContext(const KUrl& url, bool allowPro
       top = dynamic_cast<TopDUContext*>(top->importedParentContexts().first().context(0));
 
       if(!top)
+      {
         kDebug(9007) << "WARNING: Proxy-context had invalid content-context";
+      }
 
     } else {
       kDebug(9007) << "ERROR: Proxy-context has no content-context";
@@ -936,47 +647,12 @@ QWidget* CppLanguageSupport::specialLanguageObjectNavigationWidget(const KUrl& u
     return new Cpp::NavigationWidget(*m.second, preprocessedBody);
 }
 
-bool CppLanguageSupport::needsUpdate(const Cpp::EnvironmentFilePointer& file, const KUrl& localPath, const KUrl::List& includePaths) const
-{
-  if(file->needsUpdate())
-    return true;
-
-  ///@todo somehow this check should also go into EnvironmentManager
-  for( Cpp::ReferenceCountedStringSet::Iterator it = file->missingIncludeFiles().iterator(); it; ++it ) {
-
-    ///@todo store IncludeLocal/IncludeGlobal somewhere
-    QPair<KUrl, KUrl> included = findInclude( includePaths, localPath, (*it).str(), rpp::Preprocessor::IncludeLocal, KUrl(), true );
-    if(!included.first.isEmpty())
-      return true;
-  }
-
-  return false;
-}
-
 void CppLanguageSupport::newClassWizard()
 {
   //TODO: Should give some hint on where it should be added
   SimpleRefactoring::self().createNewClass(0);
 }
 
-void CppLanguageSupport::replaceCurrentAccess(KUrl url, QString old, QString _new)
-{
-  IDocument* document = ICore::self()->documentController()->documentForUrl(url);
-  if(document) {
-    KTextEditor::Document* textDocument = document->textDocument();
-    if(textDocument) {
-      KTextEditor::View* activeView = textDocument->activeView();
-      if(activeView) {
-        KTextEditor::Cursor cursor = activeView->cursorPosition();
-        KTextEditor::Range oldRange = KTextEditor::Range(cursor-KTextEditor::Cursor(0,old.length()), cursor);
-        if(oldRange.start().column() >= 0 && textDocument->text(oldRange) == old) {
-          textDocument->replaceText(oldRange, _new);
-        }
-      }
-    }
-  }
-}
-
 UIBlockTester::UIBlockTesterThread::UIBlockTesterThread( UIBlockTester& parent ) : QThread(), m_parent( parent ), m_stop(false) {
 }
 
