@@ -62,6 +62,7 @@
 #include <qwaitcondition.h>
 #include <language/duchain/duchainutils.h>
 #include "includepathcomputer.h"
+#include <interfaces/iuicontroller.h>
 
 //#define DUMP_SMART_RANGES
 //#define DUMP_AST
@@ -235,7 +236,6 @@ const QList<IndexedString>& CPPParseJob::includePaths() const {
             Q_ASSERT(!DUChain::lock()->currentThreadHasReadLock() && !DUChain::lock()->currentThreadHasWriteLock());
             m_waitForIncludePathsMutex.lock();
             qRegisterMetaType<CPPParseJob*>("CPPParseJob*");
-            ///@todo Make sure this doesn't create problems in corner cases
             QMetaObject::invokeMethod(cpp(), "findIncludePathsForJob", Qt::QueuedConnection, Q_ARG(CPPParseJob*, const_cast<CPPParseJob*>(this)));
             //Will be woken once the include-paths are computed
             m_waitForIncludePaths.wait(&m_waitForIncludePathsMutex);
@@ -396,6 +396,9 @@ void CPPInternalParseJob::initialize() {
 
 void CPPInternalParseJob::highlightIfNeeded()
 {
+    if(!KDevelop::EditorIntegrator::documentForUrl(parentJob()->document()))
+      return;
+    
     DUChainReadLocker l(DUChain::lock());
     ReferencedTopDUContext standardContext = DUChainUtils::standardContextForUrl(parentJob()->document().toUrl());
 
@@ -410,11 +413,11 @@ void CPPInternalParseJob::highlightIfNeeded()
 
 void CPPInternalParseJob::run()
 {
-    if(!CppLanguageSupport::self())
+    if(!ICore::self()->languageController()->language("C++")->languageSupport())
       return;
     //If we have a parent, the parse-mutex is already locked
     QReadLocker lock(parentJob()->parentPreprocessor() ? 0 : parentJob()->cpp()->language()->parseLock());
-    if(!CppLanguageSupport::self())
+    if(!ICore::self()->languageController()->language("C++")->languageSupport())
       return;
     
     UrlParseLock urlLock(parentJob()->document());
@@ -534,6 +537,7 @@ void CPPInternalParseJob::run()
             updatingContentContext->smartRange() &&
             updatingContentContext->parsingEnvironmentFile()->modificationRevision().modificationTime == ModificationRevision::revisionForFile(updatingContentContext->url()).modificationTime) {
           kDebug() << "not processing" << updatingContentContext->url().str() << "because of missing compound tokens";
+          ICore::self()->uiController()->showErrorMessage(i18n("Not updating duchain for %1", parentJob()->document().toUrl().fileName()), 1);
           l.unlock();
           doNotChangeDUChain = true;
           ProblemPointer problem(new Problem);
@@ -763,7 +767,9 @@ void CPPInternalParseJob::run()
         if(!context.context)
           continue;
         Q_ASSERT(context.context);
-        Q_ASSERT(contentContext->imports(context.context.data(), SimpleCursor::invalid()));
+        if(!contentContext->imports(context.context.data(), SimpleCursor::invalid())) {
+          kWarning() << "Context should be imported, but is not:" << contentContext->url().str() << " <- " << context.context->url().str();
+        }
       }
     }
 
