@@ -122,6 +122,9 @@ pp_actual pp_macro_expander::resolve_formal(IndexedString name, Stream& input)
   return pp_actual();
 }
 
+#define RETURN_IF_INPUT_BROKEN    if(input.atEnd()) { kDebug() << "too early end while expanding" << macro->name.str(); return; }
+
+
 pp_macro_expander::pp_macro_expander(pp* engine, pp_frame* frame, bool inHeaderSection)
   : m_engine(engine)
   , m_frame(frame)
@@ -252,6 +255,12 @@ void pp_macro_expander::operator()(Stream& input, Stream& output)
         for(int a = formal.count()-1; a >= 0; --a) {
           if(formal[a] == indexFromCharacter('\"') || formal[a] == indexFromCharacter('\\'))
             formal.insert(a, indexFromCharacter('\\'));
+            else if(formal[a] == indexFromCharacter('\n')) {
+              //Replace newlines with "\n"
+              formal[a] = indexFromCharacter('n');
+              formal.insert(a, indexFromCharacter('\\'));
+            }
+              
         }
 
         Stream is(&formal, inputPosition);
@@ -323,6 +332,7 @@ void pp_macro_expander::operator()(Stream& input, Stream& output)
           {
             output.appendString(*cursorIt, *textIt);
           }
+          output << ' '; //Insert a whitespace to omit implicit token merging
           output.mark(input.inputPosition());
           
           if(actual.text.isEmpty()) {
@@ -385,6 +395,7 @@ void pp_macro_expander::operator()(Stream& input, Stream& output)
               IndexedString identifier = IndexedString::fromIndex( skip_identifier(es) );
 
               output.appendString(Anchor(input.inputPosition(), true), expanded);
+              output << ' '; //Prevent implicit token merging
             }
 
             macro->hidden = false;
@@ -414,6 +425,8 @@ void pp_macro_expander::operator()(Stream& input, Stream& output)
         
         QList<pp_actual> actuals;
         ++input; // skip '('
+        
+        RETURN_IF_INPUT_BROKEN
 
         pp_macro_expander expand_actual(m_engine, m_frame);
 
@@ -449,6 +462,8 @@ void pp_macro_expander::operator()(Stream& input, Stream& output)
         while (!input.atEnd() && input == ',')
         {
           ++input; // skip ','
+          
+          RETURN_IF_INPUT_BROKEN
 
           {
             PreprocessedContents actualText;
@@ -488,6 +503,7 @@ void pp_macro_expander::operator()(Stream& input, Stream& output)
           //Move one character into the output, so we don't get an endless loop
           output << input;
           ++input;
+          
           continue;
         }
         
@@ -500,7 +516,7 @@ void pp_macro_expander::operator()(Stream& input, Stream& output)
         //Q_ASSERT(!input.atEnd() && input == ')');
 
         ++input; // skip ')'
-
+        
 #if 0 // ### enable me
         assert ((macro->variadics && macro->formals.size () >= actuals.size ())
                     || macro->formals.size() == actuals.size());
@@ -514,6 +530,8 @@ void pp_macro_expander::operator()(Stream& input, Stream& output)
         if(frame.depth >= maxMacroExpansionDepth) 
         {
           kDebug() << "reached maximum macro-expansion depth while expanding" << macro->name.str();
+          RETURN_IF_INPUT_BROKEN
+          
           output << input;
           ++input;
         }else{
@@ -523,6 +541,7 @@ void pp_macro_expander::operator()(Stream& input, Stream& output)
           Stream ms((uint*)macro->definition(), macro->definitionSize(), Anchor(input.inputPosition(), true));
           ms.setOriginalInputPosition(input.originalInputPosition());
           expand_macro(ms, output);
+          output << ' '; //Prevent implicit token merging
           macro->hidden = false;
         }
       } else {
