@@ -35,6 +35,7 @@
 #include <klocale.h>
 
 #include <language/backgroundparser/backgroundparser.h>
+#include <language/codegen/coderepresentation.h>
 #include <language/duchain/duchain.h>
 #include <language/duchain/duchainlock.h>
 #include <language/duchain/topducontext.h>
@@ -59,6 +60,7 @@
 #include "codegen/unresolvedincludeassistant.h"
 
 // #define ifDebug(x) x
+#include "cpputils.h"
 
 const uint maxIncludeDepth = 50;
 
@@ -86,7 +88,7 @@ PreprocessJob::~PreprocessJob() {
 
 KDevelop::ParsingEnvironment* PreprocessJob::createStandardEnvironment() {
     CppPreprocessEnvironment* ret = new CppPreprocessEnvironment(0, Cpp::EnvironmentFilePointer());
-    ret->merge( CppLanguageSupport::self()->standardMacros() );
+    ret->merge( CppUtils::standardMacros() );
     
     return ret;
 }
@@ -98,6 +100,8 @@ CPPParseJob * PreprocessJob::parentJob() const
 
 void PreprocessJob::foundHeaderGuard(rpp::Stream& stream, KDevelop::IndexedString guardName)
 {
+  Q_UNUSED(stream);
+  
   KDevelop::DUChainWriteLocker lock(KDevelop::DUChain::lock());
   
   m_currentEnvironment->environmentFile()->setHeaderGuard(guardName);
@@ -109,14 +113,11 @@ void PreprocessJob::foundHeaderGuard(rpp::Stream& stream, KDevelop::IndexedStrin
 
 void PreprocessJob::run()
 {
-    if(!CppLanguageSupport::self())
+    if(!ICore::self()->languageController()->language("C++")->languageSupport())
       return;
   
     //If we have a parent, that parent already has locked the parse-lock
     QReadLocker lock(parentJob()->parentPreprocessor() ? 0 : parentJob()->cpp()->language()->parseLock());
-  
-    if(!CppLanguageSupport::self())
-      return;
     
     //It seems like we cannot influence the actual thread priority in thread-weaver, so for now set it here.
     //It must be low so the GUI stays fluid.
@@ -189,7 +190,7 @@ void PreprocessJob::run()
                 KUrl::List includePaths = parentJob()->includePathUrls();
                 readLock.lock();
                 
-                needsUpdate = CppLanguageSupport::self()->needsUpdate(Cpp::EnvironmentFilePointer(cppEnv), localPath, includePaths);
+                needsUpdate = CppUtils::needsUpdate(Cpp::EnvironmentFilePointer(cppEnv), localPath, includePaths);
               }
             
             if(!needsUpdate) {
@@ -205,74 +206,9 @@ void PreprocessJob::run()
     
     if(m_secondEnvironmentFile)
       m_secondEnvironmentFile->setIncludePaths(m_firstEnvironmentFile->includePaths());
-    
-    bool readFromDisk = !parentJob()->contentsAvailableFromEditor();
-    parentJob()->setReadFromDisk(readFromDisk);
 
-    QByteArray contents;
-
-    
-    QString localFile(parentJob()->document().toUrl().toLocalFile());
-  
-    QFileInfo fileInfo( localFile );
-    
-    if ( readFromDisk )
-    {
-        QFile file( localFile );
-        if ( !file.open( QIODevice::ReadOnly ) )
-        {
-            KDevelop::ProblemPointer p(new Problem());
-            p->setSource(KDevelop::ProblemData::Disk);
-            p->setDescription(i18n( "Could not open file '%1'", localFile ));
-            switch (file.error()) {
-              case QFile::ReadError:
-                  p->setExplanation(i18n("File could not be read from."));
-                  break;
-              case QFile::OpenError:
-                  p->setExplanation(i18n("File could not be opened."));
-                  break;
-              case QFile::PermissionsError:
-                  p->setExplanation(i18n("File permissions prevent opening for read."));
-                  break;
-              default:
-                  break;
-            }
-            p->setFinalLocation(DocumentRange(parentJob()->document().str(), KTextEditor::Cursor::invalid(), KTextEditor::Cursor::invalid()));
-            p->setLocationStack(parentJob()->includeStack());
-            parentJob()->addPreprocessorProblem(p);
-
-            kWarning( 9007 ) << "Could not open file" << parentJob()->document().str() << "(path" << localFile << ")" ;
-            return ;
-        }
-
-        contents = file.readAll(); ///@todo respect local encoding settings. Currently, the file is expected to be utf-8
-        
-    //        Q_ASSERT( !contents.isEmpty() );
-        file.close();
-        m_firstEnvironmentFile->setModificationRevision( KDevelop::ModificationRevision(fileInfo.lastModified()) );
-    }
-    else
-    {
-        KTextEditor::Range range = KTextEditor::Range::invalid();
-
-        //===--- Incremental Parsing!!! yay :) ---===//
-        kDebug() << "We could have just parsed the changed ranges:";
-        foreach (KTextEditor::SmartRange* range, parentJob()->changedRanges())
-            kDebug() << *range << range->text().join("\n").left(20) << "...";
-
-        contents = parentJob()->contentsFromEditor().toUtf8();
-        m_firstEnvironmentFile->setModificationRevision( KDevelop::ModificationRevision( fileInfo.lastModified(), parentJob()->revisionToken() ) );
-    }
-    
-    ifDebug( kDebug( 9007 ) << "===-- PREPROCESSING --===> "
-    << parentJob()->document().str()
-    << "<== readFromDisk:" << readFromDisk
-    << "size:" << contents.length()
-    << endl; )
-
-    if (checkAbort())
+    if (checkAbort() || !readContents())
         return;
-    
 
     {
         ///Find a context that can be updated
@@ -299,13 +235,13 @@ void PreprocessJob::run()
     
     preprocessor.setEnvironment( m_currentEnvironment );
 
-    PreprocessedContents result = preprocessor.processFile(parentJob()->document().str(), contents);
+    PreprocessedContents result = preprocessor.processFile(parentJob()->document().str(), m_contents);
 
     if(Cpp::EnvironmentManager::matchingLevel() <= Cpp::EnvironmentManager::Naive && !m_headerSectionEnded && !m_firstEnvironmentFile->headerGuard().isEmpty()) {
       if(macroNamesAtBeginning.contains(m_firstEnvironmentFile->headerGuard())) {
         //Remove the header-guard, and re-preprocess, since we don't do real environment-management(We don't allow empty versions)
         m_currentEnvironment->removeMacro(m_firstEnvironmentFile->headerGuard());
-        result = preprocessor.processFile(parentJob()->document().str(), contents);
+        result = preprocessor.processFile(parentJob()->document().str(), m_contents);
       }
     }
     
@@ -433,7 +369,7 @@ void PreprocessJob::headerSectionEndedInternal(rpp::Stream* stream)
             KUrl localPath(parentJob()->document().str());
             localPath.setFileName(QString());
             
-            if(contentEnvironment->matchEnvironment(m_currentEnvironment) && !CppLanguageSupport::self()->needsUpdate(contentEnvironment, localPath, parentJob()->includePathUrls()) && (!parentJob()->masterJob()->needUpdateEverything() || parentJob()->masterJob()->wasUpdated(content)) && (content->parsingEnvironmentFile()->featuresSatisfied(parentJob()->minimumFeatures())) ) {
+            if(contentEnvironment->matchEnvironment(m_currentEnvironment) && !CppUtils::needsUpdate(contentEnvironment, localPath, parentJob()->includePathUrls()) && (!parentJob()->masterJob()->needUpdateEverything() || parentJob()->masterJob()->wasUpdated(content)) && (content->parsingEnvironmentFile()->featuresSatisfied(parentJob()->minimumFeatures())) ) {
                 //We can completely re-use the specialized context:
                 m_secondEnvironmentFile = dynamic_cast<Cpp::EnvironmentFile*>(content->parsingEnvironmentFile().data());
                 m_updatingEnvironmentFile = m_secondEnvironmentFile;
@@ -508,7 +444,7 @@ rpp::Stream* PreprocessJob::sourceNeeded(QString& _fileName, IncludeType type, i
     if (skipCurrentPath)
       from = parentJob()->includedFromPath();
 
-    QPair<KUrl, KUrl> included = parentJob()->cpp()->findInclude( parentJob()->includePathUrls(), localPath, fileName, type, from );
+    QPair<KUrl, KUrl> included = CppUtils::findInclude( parentJob()->includePathUrls(), localPath, fileName, type, from );
     KUrl includedFile = included.first;
     if (includedFile.isValid()) {
       
@@ -573,7 +509,7 @@ rpp::Stream* PreprocessJob::sourceNeeded(QString& _fileName, IncludeType type, i
             if(includedContext) {
               Cpp::EnvironmentFilePointer includedEnvironment(dynamic_cast<Cpp::EnvironmentFile*>(includedContext->parsingEnvironmentFile().data()));
               if( includedEnvironment ) {
-                updateNeeded = CppLanguageSupport::self()->needsUpdate(includedEnvironment, localPath, parentJob()->includePathUrls());
+                updateNeeded = CppUtils::needsUpdate(includedEnvironment, localPath, parentJob()->includePathUrls());
                 updateNeeded |= !includedEnvironment->featuresSatisfied((TopDUContext::Features)(slaveMinimumFeatures & (~TopDUContext::ForceUpdateRecursive)));
                 //Do not update again if ForceUpdate is given and the context was already updated during this run
                 updateNeeded |= (slaveMinimumFeatures & TopDUContext::ForceUpdate) && !parentJob()->masterJob()->wasUpdated(includedContext.data());
@@ -678,7 +614,7 @@ rpp::Stream* PreprocessJob::sourceNeeded(QString& _fileName, IncludeType type, i
 
 bool PreprocessJob::checkAbort()
 {
-  if(!CppLanguageSupport::self()) {
+  if(!ICore::self()->languageController()->language("C++")->languageSupport()) {
     kDebug(9007) << "Environment-manager disappeared" ;
     return true;
   }
@@ -701,10 +637,92 @@ bool PreprocessJob::checkAbort()
     return false;
 }
 
+bool PreprocessJob::readContents()
+{
+    bool readFromDisk = !parentJob()->contentsAvailableFromEditor();
+    parentJob()->setReadFromDisk(readFromDisk);
+    
+    QString localFile(parentJob()->document().toUrl().toLocalFile());
+  
+    QFileInfo fileInfo( localFile );
+    
+    if ( readFromDisk )
+    {
+        QFile file( localFile );
+        if ( !file.open( QIODevice::ReadOnly ) )
+        {
+            //Try using a code-representation, as artificial code may have been inserted
+            if(artificialCodeRepresentationExists(parentJob()->document())) {
+              CodeRepresentation::Ptr repr = createCodeRepresentation(parentJob()->document());
+              m_contents = repr->text().toUtf8();
+              kDebug() << "took contents for " << parentJob()->document().toUrl() << " from code-representation:\n" << m_contents;
+              return true;
+            }
+          
+            KDevelop::ProblemPointer p(new Problem());
+            p->setSource(KDevelop::ProblemData::Disk);
+            p->setDescription(i18n( "Could not open file '%1'", localFile ));
+            switch (file.error()) {
+              case QFile::ReadError:
+                  p->setExplanation(i18n("File could not be read from."));
+                  break;
+              case QFile::OpenError:
+                  p->setExplanation(i18n("File could not be opened."));
+                  break;
+              case QFile::PermissionsError:
+                  p->setExplanation(i18n("File permissions prevent opening for read."));
+                  break;
+              default:
+                  break;
+            }
+            p->setFinalLocation(DocumentRange(parentJob()->document().str(), KTextEditor::Cursor::invalid(), KTextEditor::Cursor::invalid()));
+            p->setLocationStack(parentJob()->includeStack());
+            parentJob()->addPreprocessorProblem(p);
+
+            kWarning( 9007 ) << "Could not open file" << parentJob()->document().str() << "(path" << localFile << ")" ;
+            return false;
+        }
+        m_contents = file.readAll(); ///@todo respect local encoding settings. Currently, the file is expected to be utf-8
+        file.close();
+        m_firstEnvironmentFile->setModificationRevision( KDevelop::ModificationRevision(fileInfo.lastModified()) );
+    }
+    else{
+        ///This has to be done, before parentJob()->revisionToken() is read, as it sets the token.
+        m_contents = parentJob()->contentsFromEditor().toUtf8();
+        m_firstEnvironmentFile->setModificationRevision( KDevelop::ModificationRevision( fileInfo.lastModified(), parentJob()->revisionToken() ) );
+    }
+    
+    ///@todo Modify parsing foronly changed ranges on editor files
+#if 0
+    //===--- Incremental Parsing!!! yay :) ---===//
+    kDebug() << "We could have just parsed the changed ranges:";
+    foreach (KTextEditor::SmartRange* range, parentJob()->changedRanges())
+        kDebug() << *range << range->text().join("\n").left(20) << "...";
+#endif
+    
+    ifDebug( kDebug( 9007 ) << "===-- PREPROCESSING --===> "
+    << parentJob()->document().str()
+    << "<== readFromDisk:" << readFromDisk
+    << "size:" << contents.length()
+    << endl; )
+    
+    return true;
+}
+
 bool PreprocessJob::success() const
 {
     return m_success;
 }
 
+KDevelop::ParsingEnvironment * PreprocessJob::m_standardEnvironment = 0;
+
+const KDevelop::ParsingEnvironment * PreprocessJob::standardEnvironment()
+{
+  if(!m_standardEnvironment)
+    m_standardEnvironment = createStandardEnvironment();
+
+  return m_standardEnvironment;
+}
+
 #include "preprocessjob.moc"
 
