@@ -27,12 +27,16 @@
 #include "ui_cmakebuilddirchooser.h"
 
 CMakeBuildDirChooser::CMakeBuildDirChooser(QWidget* parent)
-    : QWidget(parent)
+    : KDialog(parent)
 {
+    setCaption(i18n("Configure a build directory"));
+    
+//     QWidget* w= new QWidget(this);
     m_chooserUi = new Ui::CMakeBuildDirChooser;
-    m_chooserUi->setupUi( this );
+    m_chooserUi->setupUi(mainWidget());
     m_chooserUi->buildFolder->setMode(KFile::Directory|KFile::ExistingOnly);
     m_chooserUi->installPrefix->setMode(KFile::Directory|KFile::ExistingOnly);
+//     setMainWidget(w);
 
     QString cmakeBin=KStandardDirs::findExe( "cmake" );
     setCMakeBinary(KUrl(cmakeBin));
@@ -52,14 +56,14 @@ void CMakeBuildDirChooser::setSourceFolder( const KUrl& srcFolder )
     update();
 }
 
-QString CMakeBuildDirChooser::buildDirProject(const KUrl& buildDir)
+QString CMakeBuildDirChooser::buildDirProject(const KUrl& srcDir)
 {
-    QFile file(buildDir.toLocalFile()+"/CMakeCache.txt");
+    QFile file(srcDir.toLocalFile()+"/CMakeCache.txt");
 
     if (!file.open(QIODevice::ReadOnly | QIODevice::Text))
     {
         kWarning(9032) << "Something really strange happened reading CMakeCache.txt";
-        return "";
+        return QString();
     }
 
     QString ret;
@@ -68,16 +72,13 @@ QString CMakeBuildDirChooser::buildDirProject(const KUrl& buildDir)
     while (!correct && !file.atEnd())
     {
         QString line = file.readLine().trimmed();
-        if(line.startsWith('#') || line.isEmpty())
-            continue;
-
         if(line.startsWith(pLine))
         {
             correct=true;
             ret=line.mid(pLine.count());
         }
     }
-    kDebug(9042) << "The source directory for " << buildDir << "is" << ret;
+    kDebug(9042) << "The source directory for " << file.fileName() << "is" << ret;
     return ret;
 }
 
@@ -93,15 +94,15 @@ void CMakeBuildDirChooser::updated()
 //  m_chooserUi->generator->setEnabled(haveCMake);
     if(!haveCMake)
     {
-        emit status(i18n("You need to select a cmake binary"));
-        emit updated( st, "" );
+        m_chooserUi->status->setText(i18n("You need to select a cmake binary"));
+        button(Ok)->setEnabled(false);
         return;
     }
 
     bool emptyUrl=m_chooserUi->buildFolder->url().isEmpty();
     if( emptyUrl ) st |= BuildFolderEmpty;
 
-    bool alreadyCreated=false, correctProject=false, dirEmpty = false, dirExists=false;
+    bool dirEmpty = false, dirExists=false;
     QString srcDir;
     if(!emptyUrl)
     {
@@ -110,80 +111,99 @@ void CMakeBuildDirChooser::updated()
         dirEmpty=dirExists && d.count()<=2;
         if(!dirEmpty && dirExists)
         {
-            alreadyCreated=QFile::exists(m_chooserUi->buildFolder->url().toLocalFile()+"/CMakeCache.txt");
-            if(alreadyCreated)
+            bool hasCache=QFile::exists(m_chooserUi->buildFolder->url().toLocalFile()+"/CMakeCache.txt");
+            if(hasCache)
             {
-                QString srcfold=m_srcFolder.toLocalFile(KUrl::RemoveTrailingSlash);
+                QString proposed=m_srcFolder.toLocalFile(KUrl::RemoveTrailingSlash);
 
                 srcDir=buildDirProject(m_chooserUi->buildFolder->url());
                 if(!srcDir.isEmpty())
-                    correctProject= (QDir(srcDir).canonicalPath()==QDir(srcfold).canonicalPath());
+                {
+                    if(QDir(srcDir).canonicalPath()==QDir(proposed).canonicalPath())
+                    {
+                            st |= CorrectBuildDir | BuildDirCreated;
+                    }
+                }
                 else
-                    kWarning(9042) << "maybe you are trying a damaged CMakeCache.txt file";
+                {
+                    kWarning(9042) << "maybe you are trying a damaged CMakeCache.txt file. Proper: ";
+                }
             }
         }
-    }else
-    {
-        emit status(i18n("You need to specify a build directory"));
-        emit updated( st, "" );
-        return;
-    }
-
-    if( alreadyCreated ) st |= BuildDirCreated;
-    if( correctProject ) st |= CorrectProject;
-
-    if(alreadyCreated && correctProject)
-    {
-        m_chooserUi->installPrefix->setEnabled(false);
-        m_chooserUi->buildType->setEnabled(false);
-        emit status(i18n("Using an already created build directory"));
+        
+        if(m_alreadyUsed.contains(buildFolder().toLocalFile(KUrl::RemoveTrailingSlash))) {
+            st=DirAlreadyCreated;
+        }
     }
     else
     {
-        bool correct=dirEmpty || !dirExists;
+        m_chooserUi->status->setText(i18n("You need to specify a build directory"));
+        button(Ok)->setEnabled(false);
+        return;
+    }
+    
+    
+    if(st & (BuildDirCreated | CorrectBuildDir))
+    {
+        m_chooserUi->status->setText(i18n("Using an already created build directory"));
+        m_chooserUi->installPrefix->setEnabled(false);
+        m_chooserUi->buildType->setEnabled(false);
+    }
+    else
+    {
+        bool correct = (dirEmpty || !dirExists) && !(st & DirAlreadyCreated);
+        
         if(correct)
         {
             st |= CorrectBuildDir;
-            emit status(QString());
+            m_chooserUi->status->setText(QString());
         }
         else
         {
-            //Useful to prevent disasters
-            if(alreadyCreated)
-                emit status(i18n("The selected build directory is not empty"));
-            else if ( alreadyCreated && !correctProject )
-                emit status(i18n("This build directory is for %1, "
+            //Useful to explain what's going wrong
+            if(st & DirAlreadyCreated)
+                m_chooserUi->status->setText(i18n("Build directory already configured."));
+            else if (!srcDir.isEmpty())
+                m_chooserUi->status->setText(i18n("This build directory is for %1, "
                         "but the project directory is %2", srcDir, m_srcFolder.toLocalFile()));
+            else if(!dirEmpty)
+                m_chooserUi->status->setText(i18n("The selected build directory is not empty"));
         }
 
         m_chooserUi->installPrefix->setEnabled(correct);
         m_chooserUi->buildType->setEnabled(correct);
     }
-    emit updated( st, srcDir );
+    button(Ok)->setEnabled(st & CorrectBuildDir);
 }
 
 void CMakeBuildDirChooser::setCMakeBinary(const KUrl& url) 
 { 
     m_chooserUi->cmakeBin->setUrl(url); 
-    update();
+    updated();
 }
 
 void CMakeBuildDirChooser::setInstallPrefix(const KUrl& url) 
 { 
     m_chooserUi->installPrefix->setUrl(url); 
-    update();
+    updated();
 }
 
 void CMakeBuildDirChooser::setBuildFolder(const KUrl& url) 
 { 
     m_chooserUi->buildFolder->setUrl(url); 
-    update();
+    updated();
 }
 
 void CMakeBuildDirChooser::setBuildType(const QString& s) 
 { 
     m_chooserUi->buildType->addItem(s); 
-    update();
+    updated();
+}
+
+void CMakeBuildDirChooser::setAlreadyUsed (const QStringList & used)
+{
+    m_alreadyUsed = used;
+    updated();
 }
 
 KUrl CMakeBuildDirChooser::cmakeBinary() const { return m_chooserUi->cmakeBin->url(); }
