@@ -57,8 +57,8 @@
 #include <language/duchain/declaration.h>
 #include <language/duchain/duchainlock.h>
 #include <language/codecompletion/codecompletion.h>
+#include <language/codegen/applychangeswidget.h>
 
-#include "applychangeswidget.h"
 #include "cmakedocumentation.h"
 #include "cmakemodelitems.h"
 #include "cmakenavigationwidget.h"
@@ -71,6 +71,7 @@
 #include "icmakebuilder.h"
 #include "cmakeutils.h"
 #include "cmaketypes.h"
+#include "parser/cmakeparserutils.h"
 
 #ifdef CMAKEDEBUGVISITOR
 #include "cmakedebugvisitor.h"
@@ -139,15 +140,11 @@ KUrl::List resolveSystemDirs(KDevelop::IProject* project, const QStringList& dir
 }
 
 CMakeManager::CMakeManager( QObject* parent, const QVariantList& )
-    : KDevelop::IPlugin( CMakeSupportFactory::componentData(), parent ), m_builder(0)
+    : KDevelop::IPlugin( CMakeSupportFactory::componentData(), parent )
 {
     KDEV_USE_EXTENSION_INTERFACE( KDevelop::IBuildSystemManager )
     KDEV_USE_EXTENSION_INTERFACE( KDevelop::IProjectFileManager )
     KDEV_USE_EXTENSION_INTERFACE( KDevelop::ILanguageSupport )
-    IPlugin* i = core()->pluginController()->pluginForExtension( "org.kdevelop.ICMakeBuilder" );
-    Q_ASSERT(i);
-    m_builder = i->extension<ICMakeBuilder>();
-    Q_ASSERT(m_builder);
 
     new CodeCompletion(this, new CMakeCodeCompletionModel(), name());
 
@@ -179,7 +176,9 @@ KUrl CMakeManager::buildDirectory(KDevelop::ProjectBaseItem *item) const
         return path;
     }
 
-    KUrl projectPath = m_realRoot[item->project()];
+    //Can't get it from item->project()->projectItem()->url() because
+    //it might not be created yet.
+    KUrl projectPath = CMake::projectRoot(item->project());
     QString relative=KUrl::relativeUrl( projectPath, fi->url() );
     path.addPath(relative);
     path.cleanPath();
@@ -190,57 +189,10 @@ KUrl CMakeManager::buildDirectory(KDevelop::ProjectBaseItem *item) const
 
 KDevelop::ReferencedTopDUContext CMakeManager::initializeProject(KDevelop::IProject* project, const KUrl& baseUrl)
 {
-    QString cmakeCmd=KStandardDirs::findExe("cmake");
-    
-    QString versionOutput=CMake::executeProcess(cmakeCmd, QStringList("--version"));
-    QRegExp rx("([0-9]+).([0-9]+)-patch ([0-9]+)");
-    rx.indexIn(versionOutput);
-    QStringList version=rx.capturedTexts();
-    version.takeFirst();
-    
-    VariableMap m_varsDef;
-    QStringList modulePathDef=guessCMakeModulesDirectories(cmakeCmd, version);
-    m_modulePathPerProject[project]=modulePathDef;
-    kDebug(9042) << "found module path is" << modulePathDef;
-    m_varsDef.insert("CMAKE_BINARY_DIR", QStringList("#[bin_dir]"));
-    m_varsDef.insert("CMAKE_INSTALL_PREFIX", QStringList("#[install_dir]"));
-    m_varsDef.insert("CMAKE_COMMAND", QStringList(cmakeCmd));
-    m_varsDef.insert("CMAKE_MAJOR_VERSION", QStringList(version[0]));
-    m_varsDef.insert("CMAKE_MINOR_VERSION", QStringList(version[1]));
-    m_varsDef.insert("CMAKE_PATCH_VERSION", QStringList(version[2]));
-    
-    QStringList cmakeInitScripts;
-#ifdef Q_OS_WIN
-    cmakeInitScripts << "CMakeMinGWFindMake.cmake";
-    cmakeInitScripts << "CMakeMSYSFindMake.cmake";
-    cmakeInitScripts << "CMakeNMakeFindMake.cmake";
-    cmakeInitScripts << "CMakeVS8FindMake.cmake";
-#else
-    cmakeInitScripts << "CMakeUnixFindMake.cmake";
-#endif
-    cmakeInitScripts << "CMakeDetermineSystem.cmake";
-    cmakeInitScripts << "CMakeSystemSpecificInformation.cmake";
-    cmakeInitScripts << "CMakeDetermineCCompiler.cmake";
-    cmakeInitScripts << "CMakeDetermineCXXCompiler.cmake";
-
-    m_varsDef.insert("CMAKE_MODULE_PATH", modulePathDef);
-    m_varsDef.insert("CMAKE_ROOT", QStringList(guessCMakeRoot(cmakeCmd, version)));
-
-    //Defines the behaviour that can't be identified on initialization scripts
-#ifdef Q_OS_WIN32
-    m_varsDef.insert("WIN32", QStringList("1"));
-    m_varsDef.insert("CMAKE_HOST_WIN32", QStringList("1"));
-#else
-    m_varsDef.insert("UNIX", QStringList("1"));
-    m_varsDef.insert("CMAKE_HOST_UNIX", QStringList("1"));
-#endif
-#ifdef Q_OS_MAC
-    m_varsDef.insert("APPLE", QStringList("1"));
-    m_varsDef.insert("CMAKE_HOST_APPLE", QStringList("1"));
-#endif
-
+    QPair<VariableMap,QStringList> initials = CMakeParserUtils::initialVariables();
+    m_modulePathPerProject[project]=initials.first["CMAKE_MODULE_PATH"];
     m_macrosPerProject[project].clear();
-    m_varsPerProject[project]=m_varsDef;
+    m_varsPerProject[project]=initials.first;
     m_varsPerProject[project].insert("CMAKE_SOURCE_DIR", QStringList(baseUrl.toLocalFile(KUrl::RemoveTrailingSlash)));
 
     KSharedConfig::Ptr cfg = project->projectConfiguration();
@@ -259,10 +211,10 @@ KDevelop::ReferencedTopDUContext CMakeManager::initializeProject(KDevelop::IProj
         if( !l.isEmpty() )
             group.writeEntry("CMakeDir", l);
         else
-            group.writeEntry("CMakeDir", modulePathDef);
+            group.writeEntry("CMakeDir", m_modulePathPerProject[project]);
     }
     else
-        group.writeEntry("CMakeDir", modulePathDef);
+        group.writeEntry("CMakeDir", m_modulePathPerProject[project]);
 
     
     KDevelop::ReferencedTopDUContext buildstrapContext;
@@ -278,14 +230,16 @@ KDevelop::ReferencedTopDUContext CMakeManager::initializeProject(KDevelop::IProj
             buildstrapContext->clearImportedParentContexts();
             buildstrapContext->deleteChildContextsRecursively();
         }else{
-            buildstrapContext=new TopDUContext(IndexedString(buildStrapUrl), SimpleRange(0,0, 0,0));
+            IndexedString idxpath(buildStrapUrl);
+            buildstrapContext=new TopDUContext(idxpath, SimpleRange(0,0, 0,0),
+                                               new ParsingEnvironmentFile(idxpath));
             DUChain::self()->addDocumentChain(buildstrapContext);
         }
         
         Q_ASSERT(buildstrapContext);
     }
     ReferencedTopDUContext ref=buildstrapContext;
-    foreach(const QString& script, cmakeInitScripts)
+    foreach(const QString& script, initials.second)
     {
         ref = includeScript(CMakeProjectVisitor::findFile(script, m_modulePathPerProject[project], QStringList()),
                               project, ref);
@@ -312,11 +266,10 @@ KDevelop::ProjectFolderItem* CMakeManager::import( KDevelop::IProject *project )
     {
         KSharedConfig::Ptr cfg = project->projectConfiguration();
         KConfigGroup group(cfg.data(), "CMake");
-        m_subprojectRoot[project] = folderUrl;
 
         if(group.hasKey("ProjectRootRelative"))
         {
-            QString relative=group.readEntry("ProjectRootRelative");
+            QString relative=CMake::projectRootRelative(project);
             folderUrl.cd(relative);
         }
         else
@@ -339,17 +292,15 @@ KDevelop::ProjectFolderItem* CMakeManager::import( KDevelop::IProject *project )
                     return 0;
                 }
                 KUrl choice=KUrl(ui.candidates->currentItem()->text());
-                QString relative=KUrl::relativeUrl(folderUrl, choice);
-                group.writeEntry("ProjectRootRelative", relative);
+                CMake::setProjectRootRelative(project, KUrl::relativeUrl(folderUrl, choice));
                 folderUrl=choice;
             }
             else
             {
-                group.writeEntry("ProjectRootRelative", "./");
+                CMake::setProjectRootRelative(project, "./");
             }
         }
 
-        m_realRoot[project] = folderUrl;
         m_watchers[project] = new KDirWatch(project);
         m_rootItem = new CMakeFolderItem(project, folderUrl.url(), 0 );
         m_rootItem->setProjectRoot(true);
@@ -372,33 +323,8 @@ KDevelop::ProjectFolderItem* CMakeManager::import( KDevelop::IProject *project )
 KDevelop::ReferencedTopDUContext CMakeManager::includeScript(const QString& file,
                                                         KDevelop::IProject * project, ReferencedTopDUContext parent)
 {
-    kDebug(9042) << "Running cmake script: " << file;
-    CMakeFileContent f = CMakeListsParser::readCMakeFile(file);
-    if(f.isEmpty())
-    {
-        kDebug() << "There is no such file: " << file;
-        return 0;
-    }
-
-    VariableMap *vm=&m_varsPerProject[project];
-    MacroMap *mm=&m_macrosPerProject[project];
-    vm->insert("CMAKE_CURRENT_BINARY_DIR", QStringList(vm->value("CMAKE_BINARY_DIR")[0]));
-    vm->insert("CMAKE_CURRENT_LIST_FILE", QStringList(file));
-    vm->insert("CMAKE_CURRENT_SOURCE_DIR", QStringList(project->folder().toLocalFile(KUrl::RemoveTrailingSlash)));
-
-    CMakeProjectVisitor v(file, parent);
-    v.setCacheValues( &m_projectCache[project] );
-    v.setVariableMap(vm);
-    v.setMacroMap(mm);
-    v.setModulePath(m_modulePathPerProject[project]);
-    v.walk(f, 0, true);
-
-    vm->remove("CMAKE_CURRENT_LIST_FILE");
-    vm->remove("CMAKE_CURRENT_SOURCE_DIR");
-    vm->remove("CMAKE_CURRENT_BINARY_DIR");
-
     m_watchers[project]->addFile(file);
-    return v.context();
+    return CMakeParserUtils::includeScript( file, parent, &m_varsPerProject[project], &m_macrosPerProject[project], project->folder().toLocalFile(KUrl::RemoveTrailingSlash), &m_projectCache[project], m_modulePathPerProject[project]);
 }
 
 QSet<QString> removeMatches(const QString& exp, const QStringList& orig)
@@ -439,14 +365,14 @@ QList<KDevelop::ProjectFolderItem*> CMakeManager::parse( KDevelop::ProjectFolder
             
             KDevelop::ReferencedTopDUContext curr;
             if(item->parent()==0)
-                curr=initializeProject(item->project(), m_realRoot[item->project()]);
+                curr=initializeProject(item->project(), item->project()->projectItem()->url());
             else
-                curr=dynamic_cast<CMakeFolderItem*>(folder->parent())->topDUContext();
+                curr=folder->formerParent()->topDUContext();
             
             kDebug(9042) << "Adding cmake: " << cmakeListsPath << " to the model";
 
             m_watchers[item->project()]->addFile(cmakeListsPath.toLocalFile());
-            QString binDir=KUrl::relativePath(m_realRoot[folder->project()].toLocalFile(), folder->url().toLocalFile());
+            QString binDir=KUrl::relativePath(folder->project()->projectItem()->url().toLocalFile(), folder->url().toLocalFile());
             if(binDir.startsWith("./"))
                 binDir=binDir.remove(0, 2);
             
@@ -512,7 +438,7 @@ QList<KDevelop::ProjectFolderItem*> CMakeManager::parse( KDevelop::ProjectFolder
                 folder->setText(data.projectName);
             }
 
-            KUrl subroot=m_subprojectRoot[item->project()];
+            KUrl subroot=item->project()->folder();
 
             foreach (const QString& subf, data.subdirectories)
             {
@@ -532,12 +458,20 @@ QList<KDevelop::ProjectFolderItem*> CMakeManager::parse( KDevelop::ProjectFolder
                 if(subroot.isParentOf(path) || path.isParentOf(subroot))
                 {
                     entries.remove(subf);
+                    CMakeFolderItem* parent=folder;
+                    if(path.upUrl()!=folder->url())
+                        parent=0;
 
-                    CMakeFolderItem* a = new CMakeFolderItem( folder->project(), subf, folder );
+                    CMakeFolderItem* a = new CMakeFolderItem( folder->project(), subf, parent );
                     kDebug() << "folder: " << a << a->index();
                     a->setUrl(path);
                     a->setDefinitions(data.definitions);
                     folderList.append( a );
+                    
+                    if(!parent) {
+                        m_pending[path]=a;
+                        a->setFormerParent(folder);
+                    }
 
                     DescriptorAttatched* datt=static_cast<DescriptorAttatched*>(a);
                     datt->setDescriptor(data.folderDeclarations[subf]);
@@ -641,7 +575,12 @@ QList<KDevelop::ProjectFolderItem*> CMakeManager::parse( KDevelop::ProjectFolder
         {
             fileurl.adjustPath(KUrl::AddTrailingSlash);
             if(!QFile::exists(cache.toLocalFile()))
-                folderList.append(new KDevelop::ProjectFolderItem( item->project(), fileurl, item ));
+            {
+                if(m_pending.contains(fileurl))
+                    item->appendRow(m_pending.take(fileurl));
+                else
+                    folderList.append(new KDevelop::ProjectFolderItem( item->project(), fileurl, item ));
+            }
         }
         else
         {
@@ -652,18 +591,28 @@ QList<KDevelop::ProjectFolderItem*> CMakeManager::parse( KDevelop::ProjectFolder
     return folderList;
 }
 
-bool CMakeManager::reload(KDevelop::ProjectBaseItem* item)
+bool CMakeManager::reload(KDevelop::ProjectBaseItem* it)
 {
-    CMakeFolderItem* folderItem = dynamic_cast<CMakeFolderItem*>(item);
-    if (folderItem) {
-        if (item == item->project()->projectItem()) {
-            item->project()->reloadModel();
-        } else {
-            QStandardItem *parent = folderItem->parent();
-            CMakeFolderItem* fi = new CMakeFolderItem( folderItem->project(), folderItem->url().toLocalFile(), parent);
-            parent->removeRow(folderItem->row());
-            reimport(fi);
-        }
+    CMakeFolderItem* item=dynamic_cast<CMakeFolderItem*>(it->folder());
+    while(!item && it->parent()) {
+        it=dynamic_cast<ProjectBaseItem*>(it->parent());
+        item=dynamic_cast<CMakeFolderItem*>(it->folder());
+    }
+    
+    if (!item || it == it->project()->projectItem()) {
+        it->project()->reloadModel();
+    } else {
+        CMakeFolderItem* former=item->formerParent();
+        QStandardItem* parent=item->parent();
+        KUrl url=item->url();
+        IProject* project=item->project();
+        
+        item->parent()->removeRow(item->row());
+        CMakeFolderItem* fi=new CMakeFolderItem(project, url.toLocalFile(), 0);
+                                                    
+        fi->setFormerParent(former);
+        parent->appendRow(fi);
+        reimport(fi);
     }
     return true;
 }
@@ -671,7 +620,7 @@ bool CMakeManager::reload(KDevelop::ProjectBaseItem* item)
 QList<KDevelop::ProjectTargetItem*> CMakeManager::targets() const
 {
     QList<KDevelop::ProjectTargetItem*> ret;
-    foreach(IProject* p, m_realRoot.keys())
+    foreach(IProject* p, m_watchers.keys())
     {
         ret+=p->projectItem()->targetList();
     }
@@ -714,45 +663,11 @@ QHash< QString, QString > CMakeManager::defines(KDevelop::ProjectBaseItem *item
 
 KDevelop::IProjectBuilder * CMakeManager::builder(KDevelop::ProjectFolderItem *) const
 {
-    Q_ASSERT(m_builder);
-    return m_builder;
-}
-
-QString CMakeManager::guessCMakeShare(const QString& cmakeBin)
-{
-    KUrl bin(cmakeBin);
-    bin=bin.upUrl();
-    bin=bin.upUrl();
-    return bin.toLocalFile(KUrl::RemoveTrailingSlash);
-}
-
-QString CMakeManager::guessCMakeRoot(const QString & cmakeBin, const QStringList& version)
-{
-    QString ret;
-    KUrl bin(guessCMakeShare(cmakeBin));
-
-    QString versionNumber = version[0]+'.'+version[1];
-
-    bin.cd(QString("share/cmake-%1").arg(versionNumber));
-
-    ret=bin.toLocalFile(KUrl::RemoveTrailingSlash);
-    QDir d(ret);
-    if(!d.exists(ret))
-    {
-        KUrl std(bin);
-        std = std.upUrl();
-        std.cd("cmake/");
-        ret=std.toLocalFile(KUrl::RemoveTrailingSlash);
-        bin = std;
-    }
-
-    kDebug(9042) << "guessing: " << bin.toLocalFile(KUrl::RemoveTrailingSlash);
-    return ret;
-}
-
-QStringList CMakeManager::guessCMakeModulesDirectories(const QString& cmakeBin, const QStringList& version)
-{
-    return QStringList(guessCMakeRoot(cmakeBin, version)+"/Modules");
+    IPlugin* i = core()->pluginController()->pluginForExtension( "org.kdevelop.ICMakeBuilder" );
+    Q_ASSERT(i);
+    ICMakeBuilder* _builder = i->extension<ICMakeBuilder>();
+    Q_ASSERT(_builder );
+    return _builder ;
 }
 
 /*void CMakeProjectManager::parseOnly(KDevelop::IProject* project, const KUrl &url)
@@ -772,7 +687,7 @@ QStringList CMakeManager::guessCMakeModulesDirectories(const QString& cmakeBin,
         return;
     }
 
-    QString currentBinDir=KUrl::relativeUrl(m_realRoot[project], url);
+    QString currentBinDir=KUrl::relativeUrl(project->projectItem()->url(), url);
     vm->insert("CMAKE_CURRENT_BINARY_DIR", QStringList(vm->value("CMAKE_BINARY_DIR")[0]+currentBinDir));
     vm->insert("CMAKE_CURRENT_LIST_FILE", QStringList(cmakeListsPath.toLocalFile(KUrl::RemoveTrailingSlash)));
     vm->insert("CMAKE_CURRENT_SOURCE_DIR", QStringList(url.toLocalFile(KUrl::RemoveTrailingSlash)));
@@ -830,7 +745,7 @@ void CMakeManager::dirtyFile(const QString & dirty)
         CMakeFolderItem *it=static_cast<CMakeFolderItem*>(files.first()->parent());
 
         KDevelop::IProject* proj=it->project();
-        KUrl projectBaseUrl=m_realRoot[proj];
+        KUrl projectBaseUrl=proj->projectItem()->url();
         projectBaseUrl.adjustPath(KUrl::AddTrailingSlash);
 
         kDebug(9032) << "reload:" << dir << projectBaseUrl << (dir!=projectBaseUrl);
@@ -847,10 +762,7 @@ void CMakeManager::dirtyFile(const QString & dirty)
                 parseOnly(proj, current);
             }
 #endif
-            QStandardItem *parent=it->parent();
-            parent->removeRow(it->row());
-            CMakeFolderItem* fi=new CMakeFolderItem( proj, dir.toLocalFile(), parent);
-            reload(fi);
+            reload(it);
         }
         else
         {
@@ -978,7 +890,10 @@ KDevelop::ProjectFolderItem* CMakeManager::addFolder( const KUrl& folder, KDevel
     Q_ASSERT(!relative.contains("/"));
 //     CMakeFileContent f = CMakeListsParser::readCMakeFile(file);
 
-    ApplyChangesWidget e(i18n("Create a folder called '%1'.", relative), lists);
+    ApplyChangesWidget e;
+    e.setCaption(relative);
+    e.setInformation(i18n("Create a folder called '%1'.", relative));
+    e.addDocuments(IndexedString(lists), IndexedString(lists));
 
     e.document()->insertLine(e.document()->lines(), QString("add_subdirectory(%1)").arg(relative));
 
@@ -997,7 +912,7 @@ KDevelop::ProjectFolderItem* CMakeManager::addFolder( const KUrl& folder, KDevel
         QTextStream out(&f);
         out << "\n";
 
-        bool saved=e.document()->documentSave();
+        bool saved=e.applyAllChanges();
         if(!saved)
             KMessageBox::error(0, i18n("KDevelop - CMake Support"),
                                   i18n("Could not save the change."));
@@ -1015,7 +930,11 @@ bool CMakeManager::removeFolder( KDevelop::ProjectFolderItem* it)
         return true;
     }
 
-    ApplyChangesWidget e(i18n("Remove a folder called '%1'.", it->text()), lists);
+    ApplyChangesWidget e;
+    e.setCaption(it->text());
+    e.setInformation(i18n("Remove a folder called '%1'.", it->text()));
+    e.addDocuments(IndexedString(lists), IndexedString(lists));
+    
     CMakeFolderItem* cmit=static_cast<CMakeFolderItem*>(it);
     KTextEditor::Range r=cmit->descriptor().range().textRange();
     kDebug(9042) << "For " << lists << " remove " << r;
@@ -1023,7 +942,7 @@ bool CMakeManager::removeFolder( KDevelop::ProjectFolderItem* it)
 
     if(e.exec())
     {
-        bool saved=e.document()->documentSave();
+        bool saved=e.applyAllChanges();
         if(!saved)
             KMessageBox::error(0, i18n("KDevelop - CMake Support"),
                                   i18n("Could not save the change."));
@@ -1071,8 +990,8 @@ bool followUses(KTextEditor::Document* doc, SimpleRange r, const QString& name,
             for(int l=r.start.line; ;l++)
             {
                 QString line=doc->line(l);
-                int c;
-                if((c=line.indexOf(')'))>=0) {
+                int c=line.indexOf(')');
+                if(c>=0) {
                     r.end=SimpleCursor(l,c);
                     break;
                 } else if(line.isEmpty()) {
@@ -1120,14 +1039,17 @@ bool CMakeManager::removeFileFromTarget( KDevelop::ProjectFileItem* it, KDevelop
     KUrl lists=folder->url();
     lists.addPath("CMakeLists.txt");
 
-    ApplyChangesWidget e(i18n("Remove a file called '%1'.", it->text()), lists);
+    ApplyChangesWidget e;
+    e.setCaption(it->text());
+    e.setInformation(i18n("Remove a file called '%1'.", it->text()));
+    e.addDocuments(IndexedString(lists), IndexedString(lists));
 
     bool ret=followUses(e.document(), r, ' '+it->text(), lists, false);
     if(ret)
     {
         if(e.exec())
         {
-            bool saved=e.document()->documentSave();
+            bool saved=e.applyAllChanges();
             if(!saved)
                 KMessageBox::error(0, i18n("KDevelop - CMake Support"),
                                     i18n("Cannot save the change."));
@@ -1153,6 +1075,8 @@ KDevelop::ProjectFileItem* CMakeManager::addFile( const KUrl& url, KDevelop::Pro
 
 bool CMakeManager::addFileToTarget( KDevelop::ProjectFileItem* it, KDevelop::ProjectTargetItem* target)
 {
+    Q_ASSERT(!it->url().isEmpty());
+    
     QSet<QString> headerExt=QSet<QString>() << ".h" << ".hpp" << ".hxx";
     foreach(const QString& ext, headerExt)
     {
@@ -1174,12 +1098,15 @@ bool CMakeManager::addFileToTarget( KDevelop::ProjectFileItem* it, KDevelop::Pro
     KUrl lists=folder->url();
     lists.addPath("CMakeLists.txt");
 
-    ApplyChangesWidget e(i18n("Add a file called '%1' to target '%2'.", it->text(), target->text()), lists);
+    ApplyChangesWidget e;
+    e.setCaption(it->fileName());
+    e.setInformation(i18n("Add a file called '%1' to target '%2'.", it->fileName(), target->text()));
+    e.addDocuments(IndexedString(lists), IndexedString(lists));
 
-    bool ret=followUses(e.document(), r, ' '+it->text(), lists, true);
+    bool ret=followUses(e.document(), r, ' '+it->fileName(), lists, true);
 
     if(ret && e.exec())
-        ret=e.document()->documentSave();
+        ret=e.applyAllChanges();
     if(!ret)
             KMessageBox::error(0, i18n("KDevelop - CMake Support"),
                                   i18n("Cannot save the change."));
