@@ -45,6 +45,7 @@ static QString currentBuildDirKey = "CurrentBuildDir";
 static QString currentCMakeBinaryKey = "Current CMake Binary";
 static QString currentBuildTypeKey = "CurrentBuildType";
 static QString currentInstallDirKey = "CurrentInstallDir";
+static QString projectRootRelativeKey = "ProjectRootRelative";
 
 namespace CMake
 {
@@ -57,15 +58,14 @@ bool checkForNeedingConfigure( KDevelop::ProjectBaseItem* item )
 
     if( !builddir.isValid() || builddir.isEmpty() )
     {
-        KDialog choosedlg(KDevelop::ICore::self()->uiController()->activeMainWindow());
-        choosedlg.setButtons( KDialog::Ok | KDialog::Cancel );
-        choosedlg.setWindowTitle( i18n( "Choose CMake Build Directory" ) );
-        choosedlg.resize( 600, 250 );
-        CMakeBuildDirChooser bd( &choosedlg );
-        bd.setSourceFolder( item->project()->folder() );
-        choosedlg.setButtons( KDialog::Ok | KDialog::Cancel );
-        choosedlg.setMainWidget( &bd );
-        if( !choosedlg.exec() )
+        CMakeBuildDirChooser bd;
+        
+        KUrl folderUrl=item->project()->folder();
+        QString relative=CMake::projectRootRelative(item->project());
+        folderUrl.cd(relative);
+        
+        bd.setSourceFolder( folderUrl );
+        if( !bd.exec() )
         {
             return false;
         }
@@ -104,6 +104,14 @@ bool checkForNeedingConfigure( KDevelop::ProjectBaseItem* item )
     return false;
 }
 
+KUrl projectRoot(KDevelop::IProject* project)
+{
+    KUrl projectPath = project->folder();
+    bool correct=projectPath.cd(CMake::projectRootRelative(project));
+    Q_ASSERT(correct);
+    return projectPath;
+}
+
 KUrl currentBuildDir( KDevelop::IProject* project )
 {
     KConfigGroup cmakeGrp = project->projectConfiguration()->group("CMake");
@@ -128,6 +136,12 @@ KUrl currentInstallDir( KDevelop::IProject* project )
     return cmakeGrp.readEntry( currentInstallDirKey, KUrl("/usr/local") );
 }
 
+QString projectRootRelative( KDevelop::IProject* project )
+{
+    KConfigGroup cmakeGrp = project->projectConfiguration()->group("CMake");
+    return cmakeGrp.readEntry( projectRootRelativeKey, QString());
+}
+
 void setCurrentInstallDir( KDevelop::IProject* project, const KUrl& url )
 {
     KConfigGroup cmakeGrp = project->projectConfiguration()->group("CMake");
@@ -156,26 +170,11 @@ void setCurrentBuildDir( KDevelop::IProject* project, const KUrl& url )
     cmakeGrp.sync();
 }
 
-QString executeProcess(const QString& execName, const QStringList& args)
+void setProjectRootRelative( KDevelop::IProject* project, const QString& relative)
 {
-    kDebug(9042) << "Executing:" << execName << "::" << args /*<< "into" << *m_vars*/;
-
-    KProcess p;
-    p.setOutputChannelMode(KProcess::MergedChannels);
-    p.setProgram(execName, args);
-    p.start();
-
-    if(!p.waitForFinished())
-    {
-        kDebug() << "failed to execute:" << execName;
-    }
-
-    QByteArray b = p.readAllStandardOutput();
-    QString t;
-    t.prepend(b.trimmed());
-    kDebug(9042) << "executed" << execName << "<" << t;
-
-    return t;
+    KConfigGroup cmakeGrp = project->projectConfiguration()->group("CMake");
+    cmakeGrp.writeEntry( projectRootRelativeKey, relative );
+    cmakeGrp.sync();
 }
 
 }
