@@ -268,6 +268,7 @@ int CMakeProjectVisitor::visit(const ProjectAst *project)
 
 int CMakeProjectVisitor::visit( const SetTargetPropsAst * targetProps)
 {
+    kDebug(9042) << "setting target props for " << targetProps->targets() << targetProps->properties();
     foreach(const QString& tname, targetProps->targets())
     {
         foreach(const SetTargetPropsAst::PropPair& t, targetProps->properties())
@@ -278,6 +279,22 @@ int CMakeProjectVisitor::visit( const SetTargetPropsAst * targetProps)
     return 1;
 }
 
+int CMakeProjectVisitor::visit( const GetTargetPropAst * prop)
+{
+    kDebug(9042) << "getting target " << prop->target() << " prop " << prop->property() << prop->variableName();
+    if(m_props[TARGET].contains(prop->target()) && !m_props[TARGET][prop->target()].contains(prop->property())) {
+        if(prop->property().startsWith("LOCATION_") && m_props[TARGET][prop->target()].contains("IMPORTED_"+prop->property()))
+            m_props[TARGET][prop->target()][prop->property()]=m_props[TARGET][prop->target()]["IMPORTED_"+prop->property()];
+            
+//         kDebug(9032) << "unexistent property" << prop->property() << "on" << prop->target();
+    }
+//     kDebug(9042) << "current properties" << m_props[TARGET][prop->target()].keys();
+//     kDebug(9042) << "goooooot" << m_props[TARGET][prop->target()][prop->property()];
+    
+    m_vars->insert(prop->variableName(), m_props[TARGET][prop->target()][prop->property()]);
+    return 1;
+}
+
 int CMakeProjectVisitor::visit(const AddSubdirectoryAst *subd)
 {
     kDebug(9042) << "adding subdirectory" << subd->sourceDir();
@@ -291,7 +308,7 @@ int CMakeProjectVisitor::visit(const AddSubdirectoryAst *subd)
 
 int CMakeProjectVisitor::visit(const SubdirsAst *sdirs)
 {
-    kDebug(9042) << "adding subdirectory" << sdirs->directories() << sdirs->exluceFromAll();
+    kDebug(9042) << "adding subdirectories" << sdirs->directories() << sdirs->exluceFromAll();
     m_subdirectories += sdirs->directories() << sdirs->exluceFromAll();
     return 1;
 }
@@ -328,14 +345,17 @@ CMakeProjectVisitor::VisitorState CMakeProjectVisitor::stackTop() const
 
 void CMakeProjectVisitor::defineTarget(const QString& id, const QStringList& sources, Target::Type t)
 {
-    if (!m_targetForId.contains(id))
+    if (m_targetForId.contains(id))
         kDebug(9032) << "warning! there already was a target called" << id;
 
     VisitorState p=stackTop();
 
-    DUChainWriteLocker lock(DUChain::lock());
-    Declaration *d = new Declaration(p.code->at(p.line).arguments.first().range(), p.context);
-    d->setIdentifier( Identifier(id) );
+    Declaration *d=0;
+    if(!p.code->at(p.line).arguments.isEmpty()) {
+        DUChainWriteLocker lock(DUChain::lock());
+        d= new Declaration(p.code->at(p.line).arguments.first().range(), p.context);
+        d->setIdentifier( Identifier(id) );
+    }
     
     Target target;
     target.name=id;
@@ -604,7 +624,9 @@ KDevelop::ReferencedTopDUContext CMakeProjectVisitor::createContext(const KUrl&
     }
     else
     {
-        topctx=new TopDUContext(IndexedString(path), SimpleRange(0,0, endl, endc));
+        IndexedString idxpath(path);
+        topctx=new TopDUContext(idxpath, SimpleRange(0,0, endl, endc),
+                                new ParsingEnvironmentFile(idxpath));
         DUChain::self()->addDocumentChain(topctx);
 
         Q_ASSERT(DUChain::self()->chainForDocument(path));
@@ -906,25 +928,8 @@ int CMakeProjectVisitor::visit(const MacroAst *macro)
     m.name = macro->macroName();
     m.knownArgs=macro->knownArgs();
     m.isFunction=false;
-    CMakeFileContent::const_iterator it=macro->content().constBegin()+macro->line();
-    CMakeFileContent::const_iterator itEnd=macro->content().constEnd();
-    int lines=0;
-    for(; it!=itEnd; ++it, ++lines)
-    {
-        if(it->name.toLower()=="endmacro")
-            break;
-        m.code += *it;
-    }
-    ++lines; //We do not want to return to endmacro
-
-    if(it!=itEnd)
-    {
-        m_macros->insert(macro->macroName(), m);
-        macroDeclaration(macro->content()[macro->line()],
-                     macro->content()[macro->line()+lines-1],
-                     m.knownArgs);
-    }
-    return lines;
+    
+    return declareFunction(m, macro->content(), macro->line(), "endmacro");
 }
 
 int CMakeProjectVisitor::visit(const FunctionAst *func)
@@ -934,13 +939,20 @@ int CMakeProjectVisitor::visit(const FunctionAst *func)
     m.name = func->name();
     m.knownArgs=func->knownArgs();
     m.isFunction=true;
-    CMakeFileContent::const_iterator it=func->content().constBegin()+func->line();
-    CMakeFileContent::const_iterator itEnd=func->content().constEnd();
+    
+    return declareFunction(m, func->content(), func->line(), "endfunction");
+}
+
+int CMakeProjectVisitor::declareFunction(Macro m, const CMakeFileContent& content,
+                                         int initial, const QString& end)
+{
+    CMakeFileContent::const_iterator it=content.constBegin()+initial;
+    CMakeFileContent::const_iterator itEnd=content.constEnd();
 
     int lines=0;
     for(; it!=itEnd; ++it)
     {
-        if(it->name.toLower()=="endfunction")
+        if(it->name.toLower()==end)
             break;
         m.code += *it;
         ++lines;
@@ -949,11 +961,9 @@ int CMakeProjectVisitor::visit(const FunctionAst *func)
 
     if(it!=itEnd)
     {
-        m_macros->insert(func->name(), m);
+        m_macros->insert(m.name, m);
 
-        macroDeclaration(func->content()[func->line()],
-                        func->content()[func->line()+lines-1],
-                        m.knownArgs);
+        macroDeclaration(content[initial], content[initial+lines-1], m.knownArgs);
     }
     return lines;
 }
@@ -1288,13 +1298,28 @@ int CMakeProjectVisitor::visit(const FileAst *file)
         }
             break;
         case FileAst::GLOB: {
-            QString current;
-            if(file->path().isEmpty())
-                current=m_vars->value("CMAKE_CURRENT_SOURCE_DIR").first();
-            else
-                current=file->path();
-            QDir d(current);
-            QStringList matches=d.entryList(file->globbingExpressions());
+            QStringList matches;
+            QString relative=file->path();
+            foreach(const QString& glob, file->globbingExpressions())
+            {
+                QStringList globs;
+                QString current;
+                if(KUrl::isRelativeUrl(glob)) {
+                    KUrl urlGlob(glob);
+                    current=urlGlob.upUrl().path();
+                    
+                    globs.append(urlGlob.fileName());
+                } else if(!relative.isEmpty()) {
+                    current=relative;
+                    globs.append(glob);
+                } else {
+                    current=m_vars->value("CMAKE_CURRENT_SOURCE_DIR").first();
+                    globs.append(glob);
+                }
+                
+                QDir d(current);
+                matches+=d.entryList(globs);
+            }
             m_vars->insert(file->variable(), matches);
             kDebug(9042) << "file glob" << file->path() << file->globbingExpressions() << matches;
         } break;
@@ -1815,7 +1840,7 @@ int CMakeProjectVisitor::visit( const SeparateArgumentsAst * separgs )
 
 int CMakeProjectVisitor::visit(const SetPropertyAst* setp)
 {
-    kDebug() << "setprops";
+    kDebug() << "setprops" << setp->type() << setp->name() << setp->values();
     if(setp->type()==GLOBAL)
         m_props[GLOBAL][QString()][setp->name()]=setp->values();
     else
@@ -1935,7 +1960,7 @@ int CMakeProjectVisitor::walk(const CMakeFileContent & fc, int line, bool isClea
             element = new MacroCallAst;
 
         createUses(*it);
-//         kDebug(9042) << "resolving:" << it->writeBack();
+        kDebug(9042) << "resolving:" << it->writeBack();
             
         CMakeFunctionDesc func = resolveVariables(*it); //FIXME not correct in while case
         bool correct = element->parseFunctionInfo(func);
@@ -1971,10 +1996,21 @@ int CMakeProjectVisitor::walk(const CMakeFileContent & fc, int line, bool isClea
         m_vars->insert("CMAKE_CURRENT_LIST_LINE", QStringList(QString::number(it->line)));
         int lines=element->accept(this);
         line+=lines;
-        it+=lines;
         m_backtrace.top().line = line;
         m_backtrace.top().context = m_topctx;
         delete element;
+        
+        if(line>fc.count()) {
+            DUChainWriteLocker lock(DUChain::lock());
+            KSharedPtr<Problem> p(new Problem);
+            p->setDescription(i18n("Unfinished function. "));
+            p->setRange(it->nameRange());
+            m_topctx->addProblem(p);
+            
+            break;
+        }
+        
+        it+=lines;
     }
     m_backtrace.pop();
     m_topctx=aux;
